@@ -104,6 +104,9 @@ void AppConfig::set_defaults()
 
         if (get("tabs_as_menu").empty())
             set("tabs_as_menu", "0");
+
+        if (get("suppress_round_corners").empty())
+            set("suppress_round_corners", "1");
 #endif // _WIN32
 
         // remove old 'use_legacy_opengl' parameter from this config, if present
@@ -138,6 +141,9 @@ void AppConfig::set_defaults()
 
         if (get("auto_toolbar_size").empty())
             set("auto_toolbar_size", "100");
+
+        if (get("use_binary_gcode_when_supported").empty())
+            set("use_binary_gcode_when_supported", "1");
  
        if (get("notify_release").empty())
            set("notify_release", "all"); // or "none" or "release"
@@ -175,6 +181,8 @@ void AppConfig::set_defaults()
 #ifdef _WIN32
         if (get("associate_gcode").empty())
             set("associate_gcode", "0");
+        if (get("associate_bgcode").empty())
+            set("associate_bgcode", "0");
 #endif // _WIN32
     }
 
@@ -205,6 +213,9 @@ void AppConfig::set_defaults()
     if (get("allow_ip_resolve").empty())
         set("allow_ip_resolve", "1");
 
+    if (get("wifi_config_dialog_declined").empty())
+        set("wifi_config_dialog_declined", "0");
+
 #ifdef _WIN32
     if (get("use_legacy_3DConnexion").empty())
         set("use_legacy_3DConnexion", "0");
@@ -678,25 +689,7 @@ bool AppConfig::update_skein_dir(const std::string &dir)
         return false; // do not save "shapes gallery" directory
     return this->set("recent", "skein_directory", dir);
 }
-/*
-std::string AppConfig::get_last_output_dir(const std::string &alt) const
-{
-	
-    const auto it = m_storage.find("");
-    if (it != m_storage.end()) {
-        const auto it2 = it->second.find("last_output_path");
-        const auto it3 = it->second.find("remember_output_path");
-        if (it2 != it->second.end() && it3 != it->second.end() && ! it2->second.empty() && it3->second == "1")
-            return it2->second;
-    }
-    return alt;
-}
 
-void AppConfig::update_last_output_dir(const std::string &dir)
-{
-    this->set("", "last_output_path", dir);
-}
-*/
 std::string AppConfig::get_last_output_dir(const std::string& alt, const bool removable) const
 {
 	std::string s1 = (removable ? "last_output_path_removable" : "last_output_path");
