@@ -20,6 +20,10 @@ struct FillBedTask: public ArrangeTaskBase
 
     std::vector<ArrItem> selected, unselected;
 
+    // For workaround regarding "holes" when filling the bed with the same
+    // item's copies
+    std::vector<ArrItem> selected_fillers;
+
     ArrangeSettings settings;
     ExtendedBed bed;
     size_t selected_existing_count = 0;
