@@ -495,8 +495,10 @@ static void make_inner_brim(const Print                   &print,
 
     loops = union_pt_chained_outside_in(loops);
     std::reverse(loops.begin(), loops.end());
-    extrusion_entities_append_loops(brim.entities, std::move(loops), ExtrusionRole::Skirt, float(flow.mm3_per_mm()),
-                                    float(flow.width()), float(print.skirt_first_layer_height()));
+    extrusion_entities_append_loops(brim.entities, std::move(loops), 
+        ExtrusionAttributes{
+            ExtrusionRole::Skirt,
+            ExtrusionFlow{ float(flow.mm3_per_mm()), float(flow.width()), float(print.skirt_first_layer_height()) } });
 }
 
 // Produce brim lines around those objects, that have the brim enabled.
@@ -677,7 +679,9 @@ ExtrusionEntityCollection make_brim(const Print &print, PrintTryCancel try_cance
 				if (i + 1 == j && first_path.size() > 3 && first_path.front().x() == first_path.back().x() && first_path.front().y() == first_path.back().y()) {
 					auto *loop = new ExtrusionLoop();
                     brim.entities.emplace_back(loop);
-					loop->paths.emplace_back(ExtrusionRole::Skirt, float(flow.mm3_per_mm()), float(flow.width()), float(print.skirt_first_layer_height()));
+                    loop->paths.emplace_back(ExtrusionAttributes{
+                        ExtrusionRole::Skirt, 
+                        ExtrusionFlow{ float(flow.mm3_per_mm()), float(flow.width()), float(print.skirt_first_layer_height()) } });
 		            Points &points = loop->paths.front().polyline.points;
 		            points.reserve(first_path.size());
 		            for (const ClipperLib_Z::IntPoint &pt : first_path)
@@ -688,7 +692,9 @@ ExtrusionEntityCollection make_brim(const Print &print, PrintTryCancel try_cance
 			    	ExtrusionEntityCollection this_loop_trimmed;
 					this_loop_trimmed.entities.reserve(j - i);
 			    	for (; i < j; ++ i) {
-			            this_loop_trimmed.entities.emplace_back(new ExtrusionPath(ExtrusionRole::Skirt, float(flow.mm3_per_mm()), float(flow.width()), float(print.skirt_first_layer_height())));
+                        this_loop_trimmed.entities.emplace_back(new ExtrusionPath({
+                            ExtrusionRole::Skirt,
+                            ExtrusionFlow{ float(flow.mm3_per_mm()), float(flow.width()), float(print.skirt_first_layer_height()) } }));
 						const ClipperLib_Z::Path &path = *loops_trimmed_order[i].first;
 			            Points &points = dynamic_cast<ExtrusionPath*>(this_loop_trimmed.entities.back())->polyline.points;
 			            points.reserve(path.size());
@@ -704,7 +710,9 @@ ExtrusionEntityCollection make_brim(const Print &print, PrintTryCancel try_cance
 			}
 		}
     } else {
-        extrusion_entities_append_loops_and_paths(brim.entities, std::move(all_loops), ExtrusionRole::Skirt, float(flow.mm3_per_mm()), float(flow.width()), float(print.skirt_first_layer_height()));
+        extrusion_entities_append_loops_and_paths(brim.entities, std::move(all_loops), 
+            ExtrusionAttributes{ ExtrusionRole::Skirt,
+                ExtrusionFlow{ float(flow.mm3_per_mm()), float(flow.width()), float(print.skirt_first_layer_height()) } });
     }
 
     make_inner_brim(print, top_level_objects_with_brim, bottom_layers_expolygons, brim);
