@@ -395,6 +395,9 @@ static void distribute_modifiers_from_object(ModelObject* from_obj, const int in
 
     for (ModelVolume* vol : from_obj->volumes)
         if (!vol->is_model_part()) {
+            // Don't add modifiers which are processed connectors
+            if (vol->cut_info.is_connector && !vol->cut_info.is_processed)
+                continue;
             auto bb = vol->mesh().transformed_bounding_box(inst_matrix * vol->get_matrix());
             // Don't add modifiers which are not intersecting with solid parts
             if (obj1_bb.intersects(bb))
@@ -425,6 +428,8 @@ static void merge_solid_parts_inside_object(ModelObjectPtrs& objects)
                 if (mv->is_model_part() && !mv->is_cut_connector())
                     mo->delete_volume(i);
             }
+            // Ensuring that volumes start with solid parts for proper slicing
+            mo->sort_volumes(true);
         }
     }
 }
@@ -464,6 +469,12 @@ const ModelObjectPtrs& Cut::perform_by_contour(std::vector<Part> parts, int dowe
 
         // Just add Upper and Lower objects to cut_object_ptrs
         post_process(upper, lower, cut_object_ptrs);
+
+        // Now merge all model parts together:
+        merge_solid_parts_inside_object(cut_object_ptrs);
+
+        // replace initial objects in model with cut object 
+        finalize(cut_object_ptrs);
     }
     else if (volumes.size() > cut_parts_cnt) {
         // Means that object is cut with connectors
@@ -490,17 +501,18 @@ const ModelObjectPtrs& Cut::perform_by_contour(std::vector<Part> parts, int dowe
         // Add Upper and Lower objects to cut_object_ptrs
         post_process(upper, lower, cut_object_ptrs);
 
-        // Add Dowel-connectors as separate objects to cut_object_ptrs
+        // Now merge all model parts together:
+        merge_solid_parts_inside_object(cut_object_ptrs);
+
+        // replace initial objects in model with cut object
+        finalize(cut_object_ptrs);
+
+        // Add Dowel-connectors as separate objects to model
         if (cut_connectors_obj.size() >= 3)
             for (size_t id = 2; id < cut_connectors_obj.size(); id++)
-                cut_object_ptrs.push_back(cut_connectors_obj[id]);
+                m_model.add_object(*cut_connectors_obj[id]);
     }
 
-    // Now merge all model parts together:
-    merge_solid_parts_inside_object(cut_object_ptrs);
-
-    finalize(cut_object_ptrs);
-
     return m_model.objects;
 }
 
