@@ -10,10 +10,12 @@
 
 #include "libslic3r.h"
 #include "ExtrusionRole.hpp"
+#include "Flow.hpp"
 #include "Polygon.hpp"
 #include "Polyline.hpp"
 
 #include <assert.h>
+#include <optional>
 #include <string_view>
 #include <numeric>
 
@@ -62,28 +64,91 @@ public:
     virtual double total_volume() const = 0;
 };
 
-typedef std::vector<ExtrusionEntity*> ExtrusionEntitiesPtr;
+using ExtrusionEntitiesPtr = std::vector<ExtrusionEntity*>;
+
+// Const reference for ordering extrusion entities without having to modify them.
+class ExtrusionEntityReference final
+{
+public:
+    ExtrusionEntityReference() = delete;
+    ExtrusionEntityReference(const ExtrusionEntity &extrusion_entity, bool flipped) : 
+        m_extrusion_entity(&extrusion_entity), m_flipped(flipped) {}
+    ExtrusionEntityReference operator=(const ExtrusionEntityReference &rhs) 
+        { m_extrusion_entity = rhs.m_extrusion_entity; m_flipped = rhs.m_flipped; return *this; }
+
+    const ExtrusionEntity& extrusion_entity() const { return *m_extrusion_entity; }
+    template<typename Type>
+    const Type*            cast()             const { return dynamic_cast<const Type*>(m_extrusion_entity); }
+    bool                   flipped()          const { return m_flipped; }
+
+private:
+    const ExtrusionEntity *m_extrusion_entity;
+    bool                   m_flipped;
+};
+
+using ExtrusionEntityReferences = std::vector<ExtrusionEntityReference>;
+
+struct ExtrusionFlow
+{
+    ExtrusionFlow() = default;
+    ExtrusionFlow(double mm3_per_mm, float width, float height) : 
+        mm3_per_mm{ mm3_per_mm }, width{ width }, height{ height } {}
+    ExtrusionFlow(const Flow &flow) :
+        mm3_per_mm(flow.mm3_per_mm()), width(flow.width()), height(flow.height()) {}
+
+    // Volumetric velocity. mm^3 of plastic per mm of linear head motion. Used by the G-code generator.
+    double          mm3_per_mm{ -1. };
+    // Width of the extrusion, used for visualization purposes.
+    float           width{ -1.f };
+    // Height of the extrusion, used for visualization purposes.
+    float           height{ -1.f };
+};
+
+inline bool operator==(const ExtrusionFlow &lhs, const ExtrusionFlow &rhs)
+{
+    return lhs.mm3_per_mm == rhs.mm3_per_mm && lhs.width == rhs.width && lhs.height == rhs.height;
+}
+
+struct OverhangAttributes {
+    float start_distance_from_prev_layer;
+    float end_distance_from_prev_layer;
+    float proximity_to_curled_lines; //value between 0 and 1
+};
+
+struct ExtrusionAttributes : ExtrusionFlow
+{
+    ExtrusionAttributes() = default;
+    ExtrusionAttributes(ExtrusionRole role) : role{ role } {}
+    ExtrusionAttributes(ExtrusionRole role, const Flow &flow) : role{ role }, ExtrusionFlow{ flow } {}
+    ExtrusionAttributes(ExtrusionRole role, const ExtrusionFlow &flow) : role{ role }, ExtrusionFlow{ flow } {}
+
+    // What is the role / purpose of this extrusion?
+    ExtrusionRole   role{ ExtrusionRole::None };
+    // OVerhangAttributes are currently computed for perimeters if dynamic overhangs are enabled. 
+    // They are used to control fan and print speed in export.
+    std::optional<OverhangAttributes> overhang_attributes;
+};
+
+inline bool operator==(const ExtrusionAttributes &lhs, const ExtrusionAttributes &rhs)
+{
+    return static_cast<const ExtrusionFlow&>(lhs) == static_cast<const ExtrusionFlow&>(rhs) &&
+           lhs.role == rhs.role;
+}
 
 class ExtrusionPath : public ExtrusionEntity
 {
 public:
     Polyline polyline;
-    // Volumetric velocity. mm^3 of plastic per mm of linear head motion. Used by the G-code generator.
-    double mm3_per_mm;
-    // Width of the extrusion, used for visualization purposes.
-    float width;
-    // Height of the extrusion, used for visualization purposes.
-    float height;
 
-    ExtrusionPath(ExtrusionRole role) : mm3_per_mm(-1), width(-1), height(-1), m_role(role) {}
-    ExtrusionPath(ExtrusionRole role, double mm3_per_mm, float width, float height) : mm3_per_mm(mm3_per_mm), width(width), height(height), m_role(role) {}
-    ExtrusionPath(const ExtrusionPath& rhs) : polyline(rhs.polyline), mm3_per_mm(rhs.mm3_per_mm), width(rhs.width), height(rhs.height), m_role(rhs.m_role) {}
-    ExtrusionPath(ExtrusionPath&& rhs) : polyline(std::move(rhs.polyline)), mm3_per_mm(rhs.mm3_per_mm), width(rhs.width), height(rhs.height), m_role(rhs.m_role) {}
-    ExtrusionPath(const Polyline &polyline, const ExtrusionPath &rhs) : polyline(polyline), mm3_per_mm(rhs.mm3_per_mm), width(rhs.width), height(rhs.height), m_role(rhs.m_role) {}
-    ExtrusionPath(Polyline &&polyline, const ExtrusionPath &rhs) : polyline(std::move(polyline)), mm3_per_mm(rhs.mm3_per_mm), width(rhs.width), height(rhs.height), m_role(rhs.m_role) {}
+    ExtrusionPath(ExtrusionRole role) : m_attributes{ role } {}
+    ExtrusionPath(const ExtrusionAttributes &attributes) : m_attributes(attributes) {}
+    ExtrusionPath(const ExtrusionPath &rhs) : polyline(rhs.polyline), m_attributes(rhs.m_attributes) {}
+    ExtrusionPath(ExtrusionPath &&rhs) : polyline(std::move(rhs.polyline)), m_attributes(rhs.m_attributes) {}
+    ExtrusionPath(const Polyline &polyline, const ExtrusionAttributes &attribs) : polyline(polyline), m_attributes(attribs) {}
+    ExtrusionPath(Polyline &&polyline, const ExtrusionAttributes &attribs) : polyline(std::move(polyline)), m_attributes(attribs) {}
 
-    ExtrusionPath& operator=(const ExtrusionPath& rhs) { m_role = rhs.m_role; this->mm3_per_mm = rhs.mm3_per_mm; this->width = rhs.width; this->height = rhs.height; this->polyline = rhs.polyline; return *this; }
-    ExtrusionPath& operator=(ExtrusionPath&& rhs) { m_role = rhs.m_role; this->mm3_per_mm = rhs.mm3_per_mm; this->width = rhs.width; this->height = rhs.height; this->polyline = std::move(rhs.polyline); return *this; }
+    ExtrusionPath& operator=(const ExtrusionPath &rhs) { this->polyline = rhs.polyline; m_attributes = rhs.m_attributes; return *this; }
+    ExtrusionPath& operator=(ExtrusionPath &&rhs) { this->polyline = std::move(rhs.polyline); m_attributes = rhs.m_attributes; return *this; }
 
 	ExtrusionEntity* clone() const override { return new ExtrusionPath(*this); }
     // Create a new object, initialize it with this object using the move semantics.
@@ -104,35 +169,46 @@ public:
     void clip_end(double distance);
     void simplify(double tolerance);
     double length() const override;
-    ExtrusionRole role() const override { return m_role; }
+   
+    const ExtrusionAttributes&  attributes() const { return m_attributes; }
+    ExtrusionRole               role() const override { return m_attributes.role; }
+    float                       width() const { return m_attributes.width; }
+    float                       height() const { return m_attributes.height; }
+    double                      mm3_per_mm() const { return m_attributes.mm3_per_mm; }
+    // Minimum volumetric velocity of this extrusion entity. Used by the constant nozzle pressure algorithm.
+    double                      min_mm3_per_mm() const override { return m_attributes.mm3_per_mm; }
+    std::optional<OverhangAttributes>& overhang_attributes_mutable() { return m_attributes.overhang_attributes; }
+
     // Produce a list of 2D polygons covered by the extruded paths, offsetted by the extrusion width.
     // Increase the offset by scaled_epsilon to achieve an overlap, so a union will produce no gaps.
-    void polygons_covered_by_width(Polygons &out, const float scaled_epsilon) const override;
+    void        polygons_covered_by_width(Polygons &out, const float scaled_epsilon) const override;
     // Produce a list of 2D polygons covered by the extruded paths, offsetted by the extrusion spacing.
     // Increase the offset by scaled_epsilon to achieve an overlap, so a union will produce no gaps.
     // Useful to calculate area of an infill, which has been really filled in by a 100% rectilinear infill.
-    void polygons_covered_by_spacing(Polygons &out, const float scaled_epsilon) const override;
-    Polygons polygons_covered_by_width(const float scaled_epsilon = 0.f) const
+    void        polygons_covered_by_spacing(Polygons &out, const float scaled_epsilon) const override;
+    Polygons    polygons_covered_by_width(const float scaled_epsilon = 0.f) const
         { Polygons out; this->polygons_covered_by_width(out, scaled_epsilon); return out; }
-    Polygons polygons_covered_by_spacing(const float scaled_epsilon = 0.f) const
+    Polygons    polygons_covered_by_spacing(const float scaled_epsilon = 0.f) const
         { Polygons out; this->polygons_covered_by_spacing(out, scaled_epsilon); return out; }
-    // Minimum volumetric velocity of this extrusion entity. Used by the constant nozzle pressure algorithm.
-    double min_mm3_per_mm() const override { return this->mm3_per_mm; }
-    Polyline as_polyline() const override { return this->polyline; }
-    void   collect_polylines(Polylines &dst) const override { if (! this->polyline.empty()) dst.emplace_back(this->polyline); }
-    void   collect_points(Points &dst) const override { append(dst, this->polyline.points); }
-    double total_volume() const override { return mm3_per_mm * unscale<double>(length()); }
+
+    Polyline    as_polyline() const override { return this->polyline; }
+    void        collect_polylines(Polylines &dst) const override { if (! this->polyline.empty()) dst.emplace_back(this->polyline); }
+    void        collect_points(Points &dst) const override { append(dst, this->polyline.points); }
+    double      total_volume() const override { return m_attributes.mm3_per_mm * unscale<double>(length()); }
 
 private:
-    void _inflate_collection(const Polylines &polylines, ExtrusionEntityCollection* collection) const;
+    void        _inflate_collection(const Polylines &polylines, ExtrusionEntityCollection* collection) const;
 
-    ExtrusionRole m_role;
+    ExtrusionAttributes     m_attributes;
 };
 
 class ExtrusionPathOriented : public ExtrusionPath
 {
 public:
-    ExtrusionPathOriented(ExtrusionRole role, double mm3_per_mm, float width, float height) : ExtrusionPath(role, mm3_per_mm, width, height) {}
+    ExtrusionPathOriented(const ExtrusionAttributes &attribs) : ExtrusionPath(attribs) {}
+    ExtrusionPathOriented(const Polyline &polyline, const ExtrusionAttributes &attribs) : ExtrusionPath(polyline, attribs) {}
+    ExtrusionPathOriented(Polyline &&polyline, const ExtrusionAttributes &attribs) : ExtrusionPath(std::move(polyline), attribs) {}
+
     ExtrusionEntity* clone() const override { return new ExtrusionPathOriented(*this); }
     // Create a new object, initialize it with this object using the move semantics.
     ExtrusionEntity* clone_move() override { return new ExtrusionPathOriented(std::move(*this)); }
@@ -199,7 +275,8 @@ class ExtrusionLoop : public ExtrusionEntity
 public:
     ExtrusionPaths paths;
     
-    ExtrusionLoop(ExtrusionLoopRole role = elrDefault) : m_loop_role(role) {}
+    ExtrusionLoop() = default;
+    ExtrusionLoop(ExtrusionLoopRole role) : m_loop_role(role) {}
     ExtrusionLoop(const ExtrusionPaths &paths, ExtrusionLoopRole role = elrDefault) : paths(paths), m_loop_role(role) {}
     ExtrusionLoop(ExtrusionPaths &&paths, ExtrusionLoopRole role = elrDefault) : paths(std::move(paths)), m_loop_role(role) {}
     ExtrusionLoop(const ExtrusionPath &path, ExtrusionLoopRole role = elrDefault) : m_loop_role(role) 
@@ -211,16 +288,21 @@ public:
 	ExtrusionEntity* clone() const override{ return new ExtrusionLoop (*this); }
     // Create a new object, initialize it with this object using the move semantics.
 	ExtrusionEntity* clone_move() override { return new ExtrusionLoop(std::move(*this)); }
-    bool make_clockwise();
-    bool make_counter_clockwise();
-    void reverse() override;
-    const Point& first_point() const override { return this->paths.front().polyline.points.front(); }
-    const Point& last_point() const override { assert(this->first_point() == this->paths.back().polyline.points.back()); return this->first_point(); }
-    const Point& middle_point() const override { auto& path = this->paths[this->paths.size() / 2]; return path.polyline.points[path.polyline.size() / 2]; }
-    Polygon polygon() const;
-    double length() const override;
-    bool split_at_vertex(const Point &point, const double scaled_epsilon = scaled<double>(0.001));
-    void split_at(const Point &point, bool prefer_non_overhang, const double scaled_epsilon = scaled<double>(0.001));
+    double          area() const;
+    bool            is_counter_clockwise() const { return this->area() > 0; }
+    bool            is_clockwise() const { return this->area() < 0; }
+    // Reverse shall never be called on ExtrusionLoop using a virtual function call, it is most likely never what one wants,
+    // as this->can_reverse() returns false for an ExtrusionLoop.
+    void            reverse() override;
+    // Used by PerimeterGenerator to reorient extrusion loops.
+    void            reverse_loop();
+    const Point&    first_point() const override { return this->paths.front().polyline.points.front(); }
+    const Point&    last_point() const override { assert(this->first_point() == this->paths.back().polyline.points.back()); return this->first_point(); }
+    const Point&    middle_point() const override { auto& path = this->paths[this->paths.size() / 2]; return path.polyline.points[path.polyline.size() / 2]; }
+    Polygon         polygon() const;
+    double          length() const override;
+    bool            split_at_vertex(const Point &point, const double scaled_epsilon = scaled<double>(0.001));
+    void            split_at(const Point &point, bool prefer_non_overhang, const double scaled_epsilon = scaled<double>(0.001));
     struct ClosestPathPoint {
         size_t path_idx;
         size_t segment_idx;
@@ -266,59 +348,51 @@ public:
 #endif /* NDEBUG */
 
 private:
-    ExtrusionLoopRole m_loop_role;
+    ExtrusionLoopRole m_loop_role{ elrDefault };
 };
 
-inline void extrusion_paths_append(ExtrusionPaths &dst, Polylines &polylines, ExtrusionRole role, double mm3_per_mm, float width, float height)
+inline void extrusion_paths_append(ExtrusionPaths &dst, Polylines &polylines, const ExtrusionAttributes &attributes)
 {
     dst.reserve(dst.size() + polylines.size());
     for (Polyline &polyline : polylines)
-        if (polyline.is_valid()) {
-            dst.push_back(ExtrusionPath(role, mm3_per_mm, width, height));
-            dst.back().polyline = polyline;
-        }
+        if (polyline.is_valid())
+            dst.emplace_back(polyline, attributes);
 }
 
-inline void extrusion_paths_append(ExtrusionPaths &dst, Polylines &&polylines, ExtrusionRole role, double mm3_per_mm, float width, float height)
+inline void extrusion_paths_append(ExtrusionPaths &dst, Polylines &&polylines, const ExtrusionAttributes &attributes)
 {
     dst.reserve(dst.size() + polylines.size());
     for (Polyline &polyline : polylines)
-        if (polyline.is_valid()) {
-            dst.push_back(ExtrusionPath(role, mm3_per_mm, width, height));
-            dst.back().polyline = std::move(polyline);
-        }
+        if (polyline.is_valid())
+            dst.emplace_back(std::move(polyline), attributes);
     polylines.clear();
 }
 
-inline void extrusion_entities_append_paths(ExtrusionEntitiesPtr &dst, const Polylines &polylines, ExtrusionRole role, double mm3_per_mm, float width, float height, bool can_reverse = true)
+inline void extrusion_entities_append_paths(ExtrusionEntitiesPtr &dst, const Polylines &polylines, const ExtrusionAttributes &attributes, bool can_reverse = true)
 {
     dst.reserve(dst.size() + polylines.size());
     for (const Polyline &polyline : polylines)
-        if (polyline.is_valid()) {
-            ExtrusionPath* extrusion_path = can_reverse ? new ExtrusionPath(role, mm3_per_mm, width, height) : new ExtrusionPathOriented(role, mm3_per_mm, width, height);
-            dst.push_back(extrusion_path);
-            extrusion_path->polyline = polyline;
-        }
+        if (polyline.is_valid())
+            dst.emplace_back(can_reverse ? new ExtrusionPath(polyline, attributes) : new ExtrusionPathOriented(polyline, attributes));
 }
 
-inline void extrusion_entities_append_paths(ExtrusionEntitiesPtr &dst, Polylines &&polylines, ExtrusionRole role, double mm3_per_mm, float width, float height, bool can_reverse = true)
+inline void extrusion_entities_append_paths(ExtrusionEntitiesPtr &dst, Polylines &&polylines, const ExtrusionAttributes &attributes, bool can_reverse = true)
 {
     dst.reserve(dst.size() + polylines.size());
     for (Polyline &polyline : polylines)
-        if (polyline.is_valid()) {
-            ExtrusionPath *extrusion_path = can_reverse ? new ExtrusionPath(role, mm3_per_mm, width, height) : new ExtrusionPathOriented(role, mm3_per_mm, width, height);
-            dst.push_back(extrusion_path);
-            extrusion_path->polyline = std::move(polyline);
-        }
+        if (polyline.is_valid())
+            dst.emplace_back(can_reverse ?
+                new ExtrusionPath(std::move(polyline), attributes) :
+                new ExtrusionPathOriented(std::move(polyline), attributes));
     polylines.clear();
 }
 
-inline void extrusion_entities_append_loops(ExtrusionEntitiesPtr &dst, Polygons &&loops, ExtrusionRole role, double mm3_per_mm, float width, float height)
+inline void extrusion_entities_append_loops(ExtrusionEntitiesPtr &dst, Polygons &&loops, const ExtrusionAttributes &attributes)
 {
     dst.reserve(dst.size() + loops.size());
     for (Polygon &poly : loops) {
         if (poly.is_valid()) {
-            ExtrusionPath path(role, mm3_per_mm, width, height);
+            ExtrusionPath path(attributes);
             path.polyline.points = std::move(poly.points);
             path.polyline.points.push_back(path.polyline.points.front());
             dst.emplace_back(new ExtrusionLoop(std::move(path)));
@@ -327,22 +401,14 @@ inline void extrusion_entities_append_loops(ExtrusionEntitiesPtr &dst, Polygons
     loops.clear();
 }
 
-inline void extrusion_entities_append_loops_and_paths(ExtrusionEntitiesPtr &dst, Polylines &&polylines, ExtrusionRole role, double mm3_per_mm, float width, float height)
+inline void extrusion_entities_append_loops_and_paths(ExtrusionEntitiesPtr &dst, Polylines &&polylines, const ExtrusionAttributes &attributes)
 {
     dst.reserve(dst.size() + polylines.size());
-    for (Polyline &polyline : polylines) {
-        if (polyline.is_valid()) {
-            if (polyline.is_closed()) {
-                ExtrusionPath extrusion_path(role, mm3_per_mm, width, height);
-                extrusion_path.polyline = std::move(polyline);
-                dst.emplace_back(new ExtrusionLoop(std::move(extrusion_path)));
-            } else {
-                ExtrusionPath *extrusion_path = new ExtrusionPath(role, mm3_per_mm, width, height);
-                extrusion_path->polyline      = std::move(polyline);
-                dst.emplace_back(extrusion_path);
-            }
-        }
-    }
+    for (Polyline &polyline : polylines)
+        if (polyline.is_valid())
+            dst.emplace_back(polyline.is_closed() ?
+                static_cast<ExtrusionEntity*>(new ExtrusionLoop(ExtrusionPath{ std::move(polyline), attributes })) :
+                static_cast<ExtrusionEntity*>(new ExtrusionPath(std::move(polyline), attributes)));
     polylines.clear();
 }
 
