@@ -1,3 +1,7 @@
+///|/ Copyright (c) Prusa Research 2021 - 2023 Lukáš Matěna @lukasmatena, Vojtěch Bubník @bubnikv, Lukáš Hejl @hejllukas
+///|/
+///|/ PrusaSlicer is released under the terms of the AGPLv3 or higher
+///|/
 #ifndef slic3r_FillLightning_hpp_
 #define slic3r_FillLightning_hpp_
 
@@ -14,7 +18,7 @@ class Generator;
 struct GeneratorDeleter { void operator()(Generator *p); };
 using  GeneratorPtr = std::unique_ptr<Generator, GeneratorDeleter>;
 
-GeneratorPtr build_generator(const PrintObject &print_object);
+GeneratorPtr build_generator(const PrintObject &print_object, const coordf_t fill_density, const std::function<void()> &throw_on_cancel_callback);
 
 class Filler : public Slic3r::Fill
 {
@@ -24,8 +28,13 @@ public:
     Generator   *generator { nullptr };
 protected:
     Fill* clone() const override { return new Filler(*this); }
-    // Perform the fill.
-    Polylines fill_surface(const Surface *surface, const FillParams &params) override;
+
+    void _fill_surface_single(const FillParams              &params,
+                              unsigned int                   thickness_layers,
+                              const std::pair<float, Point> &direction,
+                              ExPolygon                      expolygon,
+                              Polylines &polylines_out) override;
+
     // Let the G-code export reoder the infill lines.
 	bool no_sort() const override { return false; }
 };
