@@ -1177,7 +1177,7 @@ Polyline AvoidCrossingPerimeters::travel_to(const GCodeGenerator &gcodegen, cons
     // Otherwise perform the path planning in the coordinate system of the active object.
     bool        use_external  = m_use_external_mp || m_use_external_mp_once;
     Point       scaled_origin = use_external ? Point::new_scale(gcodegen.origin()(0), gcodegen.origin()(1)) : Point(0, 0);
-    const Point start         = gcodegen.last_pos() + scaled_origin;
+    const Point start         = *gcodegen.last_position + scaled_origin;
     const Point end           = point + scaled_origin;
     const Line  travel(start, end);
 
