@@ -192,6 +192,9 @@ namespace Slic3r {
             Height,
             Width,
             Layer_Change,
+            Layer_Change_Travel,
+            Layer_Change_Retraction_Start,
+            Layer_Change_Retraction_End,
             Color_Change,
             Pause_Print,
             Custom_Code,
