@@ -1489,7 +1489,7 @@ void SeamPlacer::init(const Print &print, std::function<void(void)> throw_if_can
     }
 }
 
-void SeamPlacer::place_seam(const Layer *layer, ExtrusionLoop &loop, bool external_first,
+Point SeamPlacer::place_seam(const Layer *layer, const ExtrusionLoop &loop, bool external_first,
         const Point &last_pos) const {
     using namespace SeamPlacerImpl;
     const PrintObject *po = layer->object();
@@ -1592,7 +1592,7 @@ void SeamPlacer::place_seam(const Layer *layer, ExtrusionLoop &loop, bool extern
         //lastly, for internal perimeters, do the staggering if requested
         if (po->config().staggered_inner_seams && loop.length() > 0.0) {
             //fix depth, it is sometimes strongly underestimated
-            depth = std::max(loop.paths[projected_point.path_idx].width, depth);
+            depth = std::max(loop.paths[projected_point.path_idx].width(), depth);
 
             while (depth > 0.0f) {
                 auto next_point = get_next_loop_point(projected_point);
@@ -1610,14 +1610,7 @@ void SeamPlacer::place_seam(const Layer *layer, ExtrusionLoop &loop, bool extern
         }
     }
 
-    // Because the G-code export has 1um resolution, don't generate segments shorter than 1.5 microns,
-    // thus empty path segments will not be produced by G-code export.
-    if (!loop.split_at_vertex(seam_point, scaled<double>(0.0015))) {
-        // The point is not in the original loop.
-        // Insert it.
-        loop.split_at(seam_point, true);
-    }
-
+    return seam_point;
 }
 
 } // namespace Slic3r
