@@ -30,11 +30,16 @@ public:
         const std::string_view  comment() const
             { size_t pos = m_raw.find(';'); return (pos == std::string::npos) ? std::string_view() : std::string_view(m_raw).substr(pos + 1); }
 
+        // Return position in this->raw() string starting with the "axis" character.
+        std::string_view axis_pos(char axis) const;
         bool  has(Axis axis) const { return (m_mask & (1 << int(axis))) != 0; }
         float value(Axis axis) const { return m_axis[axis]; }
         bool  has(char axis) const;
         bool  has_value(char axis, float &value) const;
         bool  has_value(char axis, int &value) const;
+        // Parse value of an axis from raw string starting at axis_pos.
+        static bool has_value(std::string_view axis_pos, float &value);
+        static bool has_value(std::string_view axis_pos, int &value);
         float new_X(const GCodeReader &reader) const { return this->has(X) ? this->x() : reader.x(); }
         float new_Y(const GCodeReader &reader) const { return this->has(Y) ? this->y() : reader.y(); }
         float new_Z(const GCodeReader &reader) const { return this->has(Z) ? this->z() : reader.z(); }
@@ -135,7 +140,7 @@ public:
     bool parse_file(const std::string &file, callback_t callback);
     // Collect positions of line ends in the binary G-code to be used by the G-code viewer when memory mapping and displaying section of G-code
     // as an overlay in the 3D scene.
-    bool parse_file(const std::string &file, callback_t callback, std::vector<size_t> &lines_ends);
+    bool parse_file(const std::string& file, callback_t callback, std::vector<std::vector<size_t>>& lines_ends);
     // Just read the G-code file line by line, calls callback (const char *begin, const char *end). Returns false if reading the file failed.
     bool parse_file_raw(const std::string &file, raw_line_callback_t callback);
 
