@@ -6,7 +6,7 @@
 #define slic3r_VoronoiUtilsCgal_hpp_
 
 #include "Voronoi.hpp"
-#include "../Arachne/utils/VoronoiUtils.hpp"
+#include "../Arachne/utils/PolygonsSegmentIndex.hpp"
 
 namespace Slic3r::Geometry {
 class VoronoiDiagram;
@@ -18,8 +18,12 @@ public:
     static bool is_voronoi_diagram_planar_intersection(const VoronoiDiagram &voronoi_diagram);
 
     // Check if the Voronoi diagram is planar using verification that all neighboring edges are ordered CCW for each vertex.
-    static bool is_voronoi_diagram_planar_angle(const VoronoiDiagram &voronoi_diagram, const std::vector<Arachne::VoronoiUtils::Segment> &segments);
-
+    template<typename SegmentIterator>
+    static typename boost::polygon::enable_if<
+        typename boost::polygon::gtl_if<typename boost::polygon::is_segment_concept<
+            typename boost::polygon::geometry_concept<typename std::iterator_traits<SegmentIterator>::value_type>::type>::type>::type,
+        bool>::type
+    is_voronoi_diagram_planar_angle(const VoronoiDiagram &voronoi_diagram, SegmentIterator segment_begin, SegmentIterator segment_end);
 };
 } // namespace Slic3r::Geometry
 
