@@ -377,8 +377,6 @@ public:
         return false;
     }
     void                    make_perimeters();
-    // Phony version of make_fills() without parameters for Perl integration only.
-    void                    make_fills() { this->make_fills(nullptr, nullptr, nullptr); }
     void                    make_fills(FillAdaptive::Octree     *adaptive_fill_octree,
                                        FillAdaptive::Octree     *support_fill_octree,
                                        FillLightning::Generator *lightning_generator);
