@@ -1,3 +1,7 @@
+///|/ Copyright (c) Prusa Research 2021 - 2022 Tomáš Mészáros @tamasmeszaros, Lukáš Hejl @hejllukas, Vojtěch Bubník @bubnikv, Lukáš Matěna @lukasmatena
+///|/
+///|/ PrusaSlicer is released under the terms of the AGPLv3 or higher
+///|/
 #include "LocalesUtils.hpp"
 
 #ifdef _WIN32
@@ -51,16 +55,26 @@ bool is_decimal_separator_point()
     return str[1] == '.';
 }
 
-
-double string_to_double_decimal_point(const std::string_view str, size_t* pos /* = nullptr*/)
+template<class T>
+static T string_to_floating_decimal_point(const std::string_view str, size_t* pos /* = nullptr*/)
 {
-    double out;
+    T out;
     size_t p = fast_float::from_chars(str.data(), str.data() + str.size(), out).ptr - str.data();
     if (pos)
         *pos = p;
     return out;
 }
 
+double string_to_double_decimal_point(const std::string_view str, size_t* pos /* = nullptr*/)
+{
+    return string_to_floating_decimal_point<double>(str, pos);
+}
+
+float string_to_float_decimal_point(const std::string_view str, size_t* pos /* = nullptr*/)
+{
+    return string_to_floating_decimal_point<float>(str, pos);
+}
+
 std::string float_to_string_decimal_point(double value, int precision/* = -1*/)
 {
     // Our Windows build server fully supports C++17 std::to_chars. Let's use it.
