@@ -1,3 +1,7 @@
+///|/ Copyright (c) Prusa Research 2019 - 2023 Vojtěch Bubník @bubnikv, Lukáš Matěna @lukasmatena
+///|/
+///|/ PrusaSlicer is released under the terms of the AGPLv3 or higher
+///|/
 #ifndef slic3r_PolygonTrimmer_hpp_
 #define slic3r_PolygonTrimmer_hpp_
 
@@ -17,7 +21,7 @@ namespace EdgeGrid {
 
 struct TrimmedLoop
 {
-	std::vector<Point> 			points;
+	Points          			points;
 	// Number of points per segment. Empty if the loop is 
 	std::vector<unsigned int> 	segments;
 
