@@ -83,6 +83,8 @@ public:
     };
     ObsoletePresets             obsolete_presets;
 
+    std::set<std::string>       tmp_installed_presets;
+
     bool                        has_defauls_only() const 
         { return prints.has_defaults_only() && filaments.has_defaults_only() && printers.has_defaults_only(); }
 
@@ -179,6 +181,8 @@ private:
     std::vector<std::string>    merge_presets(PresetBundle &&other);
     // Update renamed_from and alias maps of system profiles.
     void 						update_system_maps();
+    // Update alias maps
+    void 						update_alias_maps();
 
     // Set the is_visible flag for filaments and sla materials,
     // apply defaults based on enabled printers when no filaments/materials are installed.
