@@ -151,16 +151,28 @@ class Integrals{
      * @param polygons List of polygons specifing the domain.
      */
     explicit Integrals(const Polygons& polygons);
+    explicit Integrals(const Polygon& polygon);
+    /**
+     * Construct integral x_i int x_i^2 (i=1,2), xy and integral 1 (area) over
+     * a set of rectangles defined by a "thick" polyline.
+     */
+    explicit Integrals(const Polylines& polylines, const std::vector<float>& widths);
 
     // TODO refactor and delete the default constructor
     Integrals() = default;
+    Integrals(float area, Vec2f x_i, Vec2f x_i_squared, float xy);
 
     float area{};
     Vec2f x_i{Vec2f::Zero()};
     Vec2f x_i_squared{Vec2f::Zero()};
     float xy{};
+
+private:
+    void add(const Integrals& other);
 };
 
+Integrals operator+(const Integrals& a, const Integrals& b);
+
 float compute_second_moment(
     const Integrals& integrals,
     const Vec2f& axis_direction
