@@ -235,12 +235,11 @@ void name_tbb_thread_pool_threads_set_locale()
 //	const size_t nthreads_hw = std::thread::hardware_concurrency();
 	const size_t nthreads_hw = tbb::this_task_arena::max_concurrency();
 	size_t       nthreads    = nthreads_hw;
+    if (thread_count) {
+        nthreads = std::min(nthreads_hw, *thread_count);
+    }
 
-#if 0
-	// Shiny profiler is not thread safe, thus disable parallelization.
-	disable_multi_threading();
-	nthreads = 1;
-#endif
+    enforce_thread_count(nthreads);
 
 	size_t                  nthreads_running(0);
 	std::condition_variable cv;
