@@ -347,36 +347,6 @@ void ConfigManipulation::toggle_print_fff_options(DynamicPrintConfig* config)
     toggle_field("thin_walls", !have_arachne);
 }
 
-void ConfigManipulation::update_print_sla_config(DynamicPrintConfig* config, const bool is_global_config/* = false*/)
-{
-    double head_penetration = config->opt_float("support_head_penetration");
-    double head_width = config->opt_float("support_head_width");
-    if (head_penetration > head_width) {
-        wxString msg_text = _(L("Head penetration should not be greater than the head width."));
-
-        MessageDialog dialog(m_msg_dlg_parent, msg_text, _(L("Invalid Head penetration")), wxICON_WARNING | wxOK);
-        DynamicPrintConfig new_conf = *config;
-        if (dialog.ShowModal() == wxID_OK) {
-            new_conf.set_key_value("support_head_penetration", new ConfigOptionFloat(head_width));
-            apply(config, &new_conf);
-        }
-    }
-
-    double pinhead_d = config->opt_float("support_head_front_diameter");
-    double pillar_d = config->opt_float("support_pillar_diameter");
-    if (pinhead_d > pillar_d) {
-        wxString msg_text = _(L("Pinhead diameter should be smaller than the pillar diameter."));
-
-        MessageDialog dialog(m_msg_dlg_parent, msg_text, _(L("Invalid pinhead diameter")), wxICON_WARNING | wxOK);
-
-        DynamicPrintConfig new_conf = *config;
-        if (dialog.ShowModal() == wxID_OK) {
-            new_conf.set_key_value("support_head_front_diameter", new ConfigOptionFloat(pillar_d / 2.0));
-            apply(config, &new_conf);
-        }
-    }
-}
-
 void ConfigManipulation::toggle_print_sla_options(DynamicPrintConfig* config)
 {
     bool supports_en = config->opt_bool("supports_enable");
