@@ -10,7 +10,7 @@
 
 class wxTextCtrl;
 class wxFlexGridSizer;
-class wxCheckBox;
+class CheckBox;
 
 namespace Slic3r {
 namespace GUI {
@@ -27,8 +27,8 @@ class ExtruderSequenceDialog: public DPIDialog
 
     wxTextCtrl* m_interval_by_layers {nullptr}; 
     wxTextCtrl* m_interval_by_mm {nullptr};
-    wxCheckBox* m_random_sequence {nullptr};
-    wxCheckBox* m_color_repetition{nullptr};
+    CheckBox* m_random_sequence {nullptr};
+    CheckBox* m_color_repetition{nullptr};
 
     wxFlexGridSizer* m_intervals_grid_sizer {nullptr};
     wxFlexGridSizer* m_extruders_grid_sizer {nullptr};
