@@ -1,3 +1,7 @@
+///|/ Copyright (c) Prusa Research 2018 - 2022 Enrico Turri @enricoturri1966, David Kocík @kocikdav, Oleksandra Iushchenko @YuSanka, Vojtěch Král @vojtechkral, Vojtěch Bubník @bubnikv
+///|/
+///|/ PrusaSlicer is released under the terms of the AGPLv3 or higher
+///|/
 #ifndef slic3r_GLToolbar_hpp_
 #define slic3r_GLToolbar_hpp_
 
@@ -61,9 +65,18 @@ public:
         Disabled,
         Hover,
         HoverPressed,
+        HoverDisabled,
         Num_States
     };
 
+    enum EHighlightState : unsigned char
+    {
+        HighlightedShown,
+        HighlightedHidden,
+        Num_Rendered_Highlight_States,
+        NotHighlighted
+    };
+
     struct Data
     {
         struct Option
@@ -103,25 +116,29 @@ private:
     EState m_state;
     Data m_data;
     EActionType m_last_action_type;
-
+    EHighlightState m_highlight_state;
 public:
     GLToolbarItem(EType type, const Data& data);
 
     EState get_state() const { return m_state; }
     void set_state(EState state) { m_state = state; }
 
+    EHighlightState get_highlight() const { return m_highlight_state; }
+    void set_highlight(EHighlightState state) { m_highlight_state = state; }
+
     const std::string& get_name() const { return m_data.name; }
     const std::string& get_icon_filename() const { return m_data.icon_filename; }
     const std::string& get_tooltip() const { return m_data.tooltip; }
     const std::string& get_additional_tooltip() const { return m_data.additional_tooltip; }
     void set_additional_tooltip(const std::string& text) { m_data.additional_tooltip = text; }
+    void set_tooltip(const std::string& text)            { m_data.tooltip = text; }
 
     void do_left_action() { m_last_action_type = Left; m_data.left.action_callback(); }
     void do_right_action() { m_last_action_type = Right; m_data.right.action_callback(); }
 
-    bool is_enabled() const { return m_state != Disabled; }
-    bool is_disabled() const { return m_state == Disabled; }
-    bool is_hovered() const { return (m_state == Hover) || (m_state == HoverPressed); }
+    bool is_enabled() const { return (m_state != Disabled) && (m_state != HoverDisabled); }
+    bool is_disabled() const { return (m_state == Disabled) || (m_state == HoverDisabled); }
+    bool is_hovered() const { return (m_state == Hover) || (m_state == HoverPressed) || (m_state == HoverDisabled); }
     bool is_pressed() const { return (m_state == Pressed) || (m_state == HoverPressed); }
     bool is_visible() const { return m_data.visible; }
     bool is_separator() const { return m_type == Separator; }
@@ -140,7 +157,7 @@ public:
     // returns true if the state changes
     bool update_enabled_state();
 
-    void render(unsigned int tex_id, float left, float right, float bottom, float top, unsigned int tex_width, unsigned int tex_height, unsigned int icon_size) const;
+    void render(const GLCanvas3D& parent, unsigned int tex_id, float left, float right, float bottom, float top, unsigned int tex_width, unsigned int tex_height, unsigned int icon_size) const;
 
 private:
     void set_visible(bool visible) { m_data.visible = visible; }
@@ -231,10 +248,11 @@ private:
     EType m_type;
     std::string m_name;
     bool m_enabled;
-    mutable GLTexture m_icons_texture;
-    mutable bool m_icons_texture_dirty;
+    GLTexture m_icons_texture;
+    bool m_icons_texture_dirty;
     BackgroundTexture m_background_texture;
-    mutable Layout m_layout;
+    GLTexture m_arrow_texture;
+    Layout m_layout;
 
     ItemsList m_items;
 
@@ -252,7 +270,6 @@ private:
     };
 
     MouseCapture m_mouse_capture;
-    std::string m_tooltip;
     int m_pressed_toggable_id;
 
 public:
@@ -261,6 +278,8 @@ public:
 
     bool init(const BackgroundTexture::Metadata& background_texture);
 
+    bool init_arrow(const std::string& filename);
+
     Layout::EType get_layout_type() const;
     void set_layout_type(Layout::EType type);
     Layout::EHorizontalOrientation get_horizontal_orientation() const { return m_layout.horizontal_orientation; }
@@ -275,14 +294,14 @@ public:
     void set_icons_size(float size);
     void set_scale(float scale);
 
-    bool is_enabled() const;
-    void set_enabled(bool enable);
+    bool is_enabled() const { return m_enabled; }
+    void set_enabled(bool enable) { m_enabled = enable; }
 
     bool add_item(const GLToolbarItem::Data& data);
     bool add_separator();
 
-    float get_width() const;
-    float get_height() const;
+    float get_width();
+    float get_height();
 
     void select_item(const std::string& name);
 
@@ -298,39 +317,43 @@ public:
     void force_left_action(int item_id, GLCanvas3D& parent) { do_action(GLToolbarItem::Left, item_id, parent, false); }
     void force_right_action(int item_id, GLCanvas3D& parent) { do_action(GLToolbarItem::Right, item_id, parent, false); }
 
-    const std::string& get_tooltip() const { return m_tooltip; }
+    std::string get_tooltip() const;
 
     void get_additional_tooltip(int item_id, std::string& text);
     void set_additional_tooltip(int item_id, const std::string& text);
+    void set_tooltip(int item_id, const std::string& text);
+    int  get_visible_items_cnt() const;
 
     // returns true if any item changed its state
     bool update_items_state();
 
-    void render(const GLCanvas3D& parent) const;    
+    void render(const GLCanvas3D& parent);
+    void render_arrow(const GLCanvas3D& parent, GLToolbarItem* highlighted_item);
 
     bool on_mouse(wxMouseEvent& evt, GLCanvas3D& parent);
-
+    // get item pointer for highlighter timer
+    GLToolbarItem* get_item(const std::string& item_name);
 private:
-    void calc_layout() const;
+    void calc_layout();
     float get_width_horizontal() const;
     float get_width_vertical() const;
     float get_height_horizontal() const;
     float get_height_vertical() const;
     float get_main_size() const;
     void do_action(GLToolbarItem::EActionType type, int item_id, GLCanvas3D& parent, bool check_hover);
-    std::string update_hover_state(const Vec2d& mouse_pos, GLCanvas3D& parent);
-    std::string update_hover_state_horizontal(const Vec2d& mouse_pos, GLCanvas3D& parent);
-    std::string update_hover_state_vertical(const Vec2d& mouse_pos, GLCanvas3D& parent);
+    void update_hover_state(const Vec2d& mouse_pos, GLCanvas3D& parent);
+    void update_hover_state_horizontal(const Vec2d& mouse_pos, GLCanvas3D& parent);
+    void update_hover_state_vertical(const Vec2d& mouse_pos, GLCanvas3D& parent);
     // returns the id of the item under the given mouse position or -1 if none
     int contains_mouse(const Vec2d& mouse_pos, const GLCanvas3D& parent) const;
     int contains_mouse_horizontal(const Vec2d& mouse_pos, const GLCanvas3D& parent) const;
     int contains_mouse_vertical(const Vec2d& mouse_pos, const GLCanvas3D& parent) const;
 
-    void render_background(float left, float top, float right, float bottom, float border) const;
-    void render_horizontal(const GLCanvas3D& parent) const;
-    void render_vertical(const GLCanvas3D& parent) const;
+    void render_background(float left, float top, float right, float bottom, float border_w, float border_h) const;
+    void render_horizontal(const GLCanvas3D& parent);
+    void render_vertical(const GLCanvas3D& parent);
 
-    bool generate_icons_texture() const;
+    bool generate_icons_texture();
 
     // returns true if any item changed its state
     bool update_items_visibility();
