@@ -1,3 +1,7 @@
+///|/ Copyright (c) Prusa Research 2018 - 2023 Enrico Turri @enricoturri1966, Filip Sykala @Jony01, Vojtěch Bubník @bubnikv, Oleksandra Iushchenko @YuSanka, Lukáš Matěna @lukasmatena, Vojtěch Král @vojtechkral
+///|/
+///|/ PrusaSlicer is released under the terms of the AGPLv3 or higher
+///|/
 #ifndef slic3r_GUI_Preview_hpp_
 #define slic3r_GUI_Preview_hpp_
 
@@ -80,16 +84,6 @@ class Preview : public wxPanel
     wxBoxSizer* m_left_sizer { nullptr };
     wxBoxSizer* m_layers_slider_sizer { nullptr };
     wxPanel* m_bottom_toolbar_panel { nullptr };
-    wxStaticText* m_label_view_type { nullptr };
-#ifdef _WIN32
-    BitmapComboBox* m_choice_view_type { nullptr };
-#else
-    wxComboBox* m_choice_view_type { nullptr };
-#endif
-    wxStaticText* m_label_show { nullptr };
-    wxComboCtrl* m_combochecklist_features { nullptr };
-    size_t m_combochecklist_features_pos { 0 };
-    wxComboCtrl* m_combochecklist_options { nullptr };
 
     DynamicPrintConfig* m_config;
     BackgroundSlicingProcess* m_process;
@@ -124,9 +118,9 @@ public:
         ColorChanges,
         PausePrints,
         CustomGCodes,
+        CenterOfGravity,
         Shells,
         ToolMarker,
-        Legend
     };
 
     Preview(wxWindow* parent, Bed3D& bed, Model* model, DynamicPrintConfig* config, BackgroundSlicingProcess* process, 
@@ -142,6 +136,7 @@ public:
     void select_view(const std::string& direction);
     void set_drop_target(wxDropTarget* target);
 
+    void load_gcode_shells();
     void load_print(bool keep_z_range = false);
     void reload_print(bool keep_volumes = false);
     void refresh_print();
@@ -154,12 +149,15 @@ public:
 
     bool is_loaded() const { return m_loaded; }
 
-    void update_bottom_toolbar();
     void update_moves_slider();
     void enable_moves_slider(bool enable);
     void move_moves_slider(wxKeyEvent& evt);
     void hide_layers_slider();
 
+    void set_keep_current_preview_type(bool value) { m_keep_current_preview_type = value; }
+
+    void set_layers_slider_values_range(int bottom, int top);
+
 private:
     bool init(wxWindow* parent, Bed3D& bed, Model* model);
 
@@ -167,9 +165,6 @@ private:
     void unbind_event_handlers();
 
     void on_size(wxSizeEvent& evt);
-    void on_choice_view_type(wxCommandEvent& evt);
-    void on_combochecklist_features(wxCommandEvent& evt);
-    void on_combochecklist_options(wxCommandEvent& evt);
 
     // Create/Update/Reset double slider on 3dPreview
     wxBoxSizer* create_layers_slider_sizer();
@@ -186,7 +181,6 @@ private:
 
     void on_layers_slider_scroll_changed(wxCommandEvent& event);
     void on_moves_slider_scroll_changed(wxCommandEvent& event);
-    wxString get_option_type_string(OptionType type) const;
 };
 
 } // namespace GUI
