@@ -1,9 +1,13 @@
+///|/ Copyright (c) Prusa Research 2019 - 2023 Pavel Mikuš @Godrak, Lukáš Matěna @lukasmatena, Enrico Turri @enricoturri1966, Vojtěch Bubník @bubnikv, Lukáš Hejl @hejllukas, Filip Sykala @Jony01
+///|/
+///|/ PrusaSlicer is released under the terms of the AGPLv3 or higher
+///|/
 #ifndef slic3r_GLGizmoPainterBase_hpp_
 #define slic3r_GLGizmoPainterBase_hpp_
 
 #include "GLGizmoBase.hpp"
 
-#include "slic3r/GUI/3DScene.hpp"
+#include "slic3r/GUI/GLModel.hpp"
 
 #include "libslic3r/ObjectID.hpp"
 #include "libslic3r/TriangleSelector.hpp"
@@ -12,6 +16,8 @@
 #include <cereal/types/vector.hpp>
 #include <GL/glew.h>
 
+#include <memory>
+#include <wx/string.h>
 
 
 namespace Slic3r::GUI {
@@ -20,6 +26,7 @@ enum class SLAGizmoEventType : unsigned char;
 class ClippingPlane;
 struct Camera;
 class GLGizmoMmuSegmentation;
+class Selection;
 
 enum class PainterGizmoType {
     FDM_SUPPORTS,
@@ -27,76 +34,43 @@ enum class PainterGizmoType {
     MMU_SEGMENTATION
 };
 
-class GLPaintContour
-{
-public:
-    GLPaintContour() = default;
-
-    void render() const;
-
-    inline bool has_VBO() const { return this->m_contour_EBO_id != 0; }
-
-    // Release the geometry data, release OpenGL VBOs.
-    void release_geometry();
-
-    // Finalize the initialization of the contour geometry and the indices, upload both to OpenGL VBO objects
-    // and possibly releasing it if it has been loaded into the VBOs.
-    void finalize_geometry();
-
-    void clear()
-    {
-        this->contour_vertices.clear();
-        this->contour_indices.clear();
-        this->contour_indices_size = 0;
-    }
-
-    std::vector<float> contour_vertices;
-    std::vector<int>   contour_indices;
-
-    // When the triangle indices are loaded into the graphics card as Vertex Buffer Objects,
-    // the above mentioned std::vectors are cleared and the following variables keep their original length.
-    size_t contour_indices_size{0};
-
-    // IDs of the Vertex Array Objects, into which the geometry has been loaded.
-    // Zero if the VBOs are not sent to GPU yet.
-    GLuint m_contour_VBO_id{0};
-    GLuint m_contour_EBO_id{0};
-};
-
 class TriangleSelectorGUI : public TriangleSelector {
 public:
     explicit TriangleSelectorGUI(const TriangleMesh& mesh)
         : TriangleSelector(mesh) {}
     virtual ~TriangleSelectorGUI() = default;
 
-    // Render current selection. Transformation matrices are supposed
-    // to be already set.
-    virtual void render(ImGuiWrapper *imgui);
-    void         render() { this->render(nullptr); }
+    virtual void render(ImGuiWrapper* imgui, const Transform3d& matrix);
+    void         render(const Transform3d& matrix) { this->render(nullptr, matrix); }
 
-    void request_update_render_data() { m_update_render_data = true; };
+    void request_update_render_data() { m_update_render_data = true; }
 
 #ifdef PRUSASLICER_TRIANGLE_SELECTOR_DEBUG
     void render_debug(ImGuiWrapper* imgui);
     bool m_show_triangles{false};
     bool m_show_invalid{false};
-#endif
+#endif // PRUSASLICER_TRIANGLE_SELECTOR_DEBUG
 
 protected:
     bool m_update_render_data = false;
 
-    static std::array<float, 4> get_seed_fill_color(const std::array<float, 4> &base_color);
+    static ColorRGBA get_seed_fill_color(const ColorRGBA& base_color);
 
 private:
     void update_render_data();
 
-    GLIndexedVertexArray                m_iva_enforcers;
-    GLIndexedVertexArray                m_iva_blockers;
-    std::array<GLIndexedVertexArray, 3> m_iva_seed_fills;
-    std::array<GLIndexedVertexArray, 3> m_varrays;
+    GLModel                m_iva_enforcers;
+    GLModel                m_iva_blockers;
+    std::array<GLModel, 3> m_iva_seed_fills;
+#ifdef PRUSASLICER_TRIANGLE_SELECTOR_DEBUG
+    std::array<GLModel, 3> m_varrays;
+#endif // PRUSASLICER_TRIANGLE_SELECTOR_DEBUG
 
 protected:
-    GLPaintContour                      m_paint_contour;
+    GLModel m_paint_contour;
+
+    void update_paint_contour();
+    void render_paint_contour(const Transform3d& matrix);
 };
 
 
@@ -109,33 +83,42 @@ private:
     ObjectID m_old_mo_id;
     size_t m_old_volumes_size = 0;
     void on_render() override {}
-    void on_render_for_picking() override {}
+
 public:
     GLGizmoPainterBase(GLCanvas3D& parent, const std::string& icon_filename, unsigned int sprite_id);
-    ~GLGizmoPainterBase() override = default;
-    virtual void set_painter_gizmo_data(const Selection& selection);
+    ~GLGizmoPainterBase() override;
+    void data_changed(bool is_serializing) override;
     virtual bool gizmo_event(SLAGizmoEventType action, const Vec2d& mouse_position, bool shift_down, bool alt_down, bool control_down);
 
     // Following function renders the triangles and cursor. Having this separated
     // from usual on_render method allows to render them before transparent
     // objects, so they can be seen inside them. The usual on_render is called
     // after all volumes (including transparent ones) are rendered.
-    virtual void render_painter_gizmo() const = 0;
+    virtual void render_painter_gizmo() = 0;
 
     virtual const float get_cursor_radius_min() const { return CursorRadiusMin; }
     virtual const float get_cursor_radius_max() const { return CursorRadiusMax; }
     virtual const float get_cursor_radius_step() const { return CursorRadiusStep; }
 
+    /// <summary>
+    /// Implement when want to process mouse events in gizmo
+    /// Click, Right click, move, drag, ...
+    /// </summary>
+    /// <param name="mouse_event">Keep information about mouse click</param>
+    /// <returns>Return True when use the information and don't want to
+    /// propagate it otherwise False.</returns>
+    bool on_mouse(const wxMouseEvent &mouse_event) override;
+
 protected:
     virtual void render_triangles(const Selection& selection) const;
-    void render_cursor() const;
-    void render_cursor_circle() const;
+    void render_cursor();
+    void render_cursor_circle();
     void render_cursor_sphere(const Transform3d& trafo) const;
     virtual void update_model_object() const = 0;
     virtual void update_from_model_object() = 0;
 
-    virtual std::array<float, 4> get_cursor_sphere_left_button_color() const { return {0.f, 0.f, 1.f, 0.25f}; }
-    virtual std::array<float, 4> get_cursor_sphere_right_button_color() const { return {1.f, 0.f, 0.f, 0.25f}; }
+    virtual ColorRGBA get_cursor_sphere_left_button_color() const  { return { 0.0f, 0.0f, 1.0f, 0.25f }; }
+    virtual ColorRGBA get_cursor_sphere_right_button_color() const { return { 1.0f, 0.0f, 0.0f, 0.25f }; }
 
     virtual EnforcerBlockerType get_left_button_state_type() const { return EnforcerBlockerType::ENFORCER; }
     virtual EnforcerBlockerType get_right_button_state_type() const { return EnforcerBlockerType::BLOCKER; }
@@ -170,6 +153,12 @@ protected:
     bool     m_paint_on_overhangs_only          = false;
     float    m_highlight_by_angle_threshold_deg = 0.f;
 
+    GLModel m_circle;
+#if !ENABLE_GL_CORE_PROFILE
+    Vec2d m_old_center{ Vec2d::Zero() };
+#endif // !ENABLE_GL_CORE_PROFILE
+    float m_old_cursor_radius{ 0.0f };
+
     static constexpr float SmartFillAngleMin  = 0.0f;
     static constexpr float SmartFillAngleMax  = 90.f;
     static constexpr float SmartFillAngleStep = 1.f;
@@ -202,7 +191,7 @@ private:
                               const Camera& camera,
                               const std::vector<Transform3d>& trafo_matrices) const;
 
-    GLIndexedVertexArray m_vbo_sphere;
+    static std::shared_ptr<GLModel> s_sphere;
 
     bool m_internal_stack_active = false;
     bool m_schedule_update = false;
@@ -224,9 +213,6 @@ private:
 
 protected:
     void on_set_state() override;
-    void on_start_dragging() override {}
-    void on_stop_dragging() override {}
-
     virtual void on_opening() = 0;
     virtual void on_shutdown() = 0;
     virtual PainterGizmoType get_painter_type() const = 0;
