@@ -1,3 +1,7 @@
+///|/ Copyright (c) Prusa Research 2020 - 2023 Lukáš Matěna @lukasmatena, Oleksandra Iushchenko @YuSanka, Enrico Turri @enricoturri1966, Tomáš Mészáros @tamasmeszaros, Vojtěch Bubník @bubnikv
+///|/
+///|/ PrusaSlicer is released under the terms of the AGPLv3 or higher
+///|/
 #include "GLGizmosCommon.hpp"
 
 #include <cassert>
@@ -23,7 +27,7 @@ CommonGizmosDataPool::CommonGizmosDataPool(GLCanvas3D* canvas)
     using c = CommonGizmosDataID;
     m_data[c::SelectionInfo].reset(   new SelectionInfo(this));
     m_data[c::InstancesHider].reset(  new InstancesHider(this));
-    m_data[c::HollowedMesh].reset(    new HollowedMesh(this));
+//    m_data[c::HollowedMesh].reset(    new HollowedMesh(this));
     m_data[c::Raycaster].reset(       new Raycaster(this));
     m_data[c::ObjectClipper].reset(   new ObjectClipper(this));
     m_data[c::SupportsClipper].reset( new SupportsClipper(this));
@@ -59,13 +63,6 @@ InstancesHider* CommonGizmosDataPool::instances_hider() const
     return inst_hider->is_valid() ? inst_hider : nullptr;
 }
 
-HollowedMesh* CommonGizmosDataPool::hollowed_mesh() const
-{
-    HollowedMesh* hol_mesh = dynamic_cast<HollowedMesh*>(m_data.at(CommonGizmosDataID::HollowedMesh).get());
-    assert(hol_mesh);
-    return hol_mesh->is_valid() ? hol_mesh : nullptr;
-}
-
 Raycaster* CommonGizmosDataPool::raycaster() const
 {
     Raycaster* rc = dynamic_cast<Raycaster*>(m_data.at(CommonGizmosDataID::Raycaster).get());
@@ -117,23 +114,35 @@ bool CommonGizmosDataPool::check_dependencies(CommonGizmosDataID required) const
 void SelectionInfo::on_update()
 {
     const Selection& selection = get_pool()->get_canvas()->get_selection();
+
+    m_model_object = nullptr;
+    m_print_object = nullptr;
+
     if (selection.is_single_full_instance()) {
         m_model_object = selection.get_model()->objects[selection.get_object_idx()];
-        m_z_shift = selection.get_volume(*selection.get_volume_idxs().begin())->get_sla_shift_z();
+        if (m_model_object)
+            m_print_object = get_pool()->get_canvas()->sla_print()->get_print_object_by_model_object_id(m_model_object->id());
+
+        m_z_shift = m_print_object ? m_print_object->get_current_elevation() : selection.get_first_volume()->get_sla_shift_z();
     }
-    else
-        m_model_object = nullptr;
 }
 
 void SelectionInfo::on_release()
 {
     m_model_object = nullptr;
+    m_model_volume = nullptr;
+}
+
+ModelInstance *SelectionInfo::model_instance() const
+{
+    int inst_idx = get_active_instance();
+    return inst_idx < int(m_model_object->instances.size()) ?
+               m_model_object->instances[get_active_instance()] : nullptr;
 }
 
 int SelectionInfo::get_active_instance() const
 {
-    const Selection& selection = get_pool()->get_canvas()->get_selection();
-    return selection.get_instance_idx();
+    return get_pool()->get_canvas()->get_selection().get_instance_idx();
 }
 
 
@@ -148,8 +157,10 @@ void InstancesHider::on_update()
 
     if (mo && active_inst != -1) {
         canvas->toggle_model_objects_visibility(false);
-        canvas->toggle_model_objects_visibility(true, mo, active_inst);
-        canvas->toggle_sla_auxiliaries_visibility(m_show_supports, mo, active_inst);
+        if (!m_hide_full_scene) {
+            canvas->toggle_model_objects_visibility(true, mo, active_inst);
+            canvas->toggle_sla_auxiliaries_visibility(false, mo, active_inst);
+        }
         canvas->set_use_clipping_planes(true);
         // Some objects may be sinking, do not show whatever is below the bed.
         canvas->set_clipping_plane(0, ClippingPlane(Vec3d::UnitZ(), -SINKING_Z_THRESHOLD));
@@ -165,7 +176,7 @@ void InstancesHider::on_update()
             for (const TriangleMesh* mesh : meshes) {
                 m_clippers.emplace_back(new MeshClipper);
                 m_clippers.back()->set_plane(ClippingPlane(-Vec3d::UnitZ(), -SINKING_Z_THRESHOLD));
-                m_clippers.back()->set_mesh(*mesh);
+                m_clippers.back()->set_mesh(mesh->its);
             }
             m_old_meshes = meshes;
         }
@@ -182,9 +193,10 @@ void InstancesHider::on_release()
     m_clippers.clear();
 }
 
-void InstancesHider::show_supports(bool show) {
-    if (m_show_supports != show) {
-        m_show_supports = show;
+void InstancesHider::set_hide_full_scene(bool hide)
+{
+    if (m_hide_full_scene != hide) {
+        m_hide_full_scene = hide;
         on_update();
     }
 }
@@ -209,131 +221,76 @@ void InstancesHider::render_cut() const
             ClippingPlane clp = *get_pool()->object_clipper()->get_clipping_plane();
             clp.set_normal(-clp.get_normal());
             clipper->set_limiting_plane(clp);
-        } else
+        }
+        else
             clipper->set_limiting_plane(ClippingPlane::ClipsNothing());
 
-        glsafe(::glPushMatrix());
-        if (mv->is_model_part())
-            glsafe(::glColor3f(0.8f, 0.3f, 0.0f));
-        else {
-            const std::array<float, 4>& c = color_from_model_volume(*mv);
-            glsafe(::glColor4f(c[0], c[1], c[2], c[3]));
-        }
+#if ENABLE_GL_CORE_PROFILE || ENABLE_OPENGL_ES
+        bool depth_test_enabled = ::glIsEnabled(GL_DEPTH_TEST);
+#else
         glsafe(::glPushAttrib(GL_DEPTH_TEST));
+#endif // ENABLE_GL_CORE_PROFILE || ENABLE_OPENGL_ES
         glsafe(::glDisable(GL_DEPTH_TEST));
-        clipper->render_cut();
+        clipper->render_cut(mv->is_model_part() ? ColorRGBA(0.8f, 0.3f, 0.0f, 1.0f) : color_from_model_volume(*mv));
+#if ENABLE_GL_CORE_PROFILE || ENABLE_OPENGL_ES
+        if (depth_test_enabled)
+            glsafe(::glEnable(GL_DEPTH_TEST));
+#else
         glsafe(::glPopAttrib());
-        glsafe(::glPopMatrix());
+#endif // ENABLE_GL_CORE_PROFILE || ENABLE_OPENGL_ES
 
         ++clipper_id;
     }
 }
 
 
-
-void HollowedMesh::on_update()
-{
-    const ModelObject* mo = get_pool()->selection_info()->model_object();
-    bool is_sla = wxGetApp().preset_bundle->printers.get_selected_preset().printer_technology() == ptSLA;
-    if (! mo || ! is_sla)
-        return;
-
-    const GLCanvas3D* canvas = get_pool()->get_canvas();
-    const PrintObjects& print_objects = canvas->sla_print()->objects();
-    const SLAPrintObject* print_object = m_print_object_idx != -1
-            ? print_objects[m_print_object_idx]
-            : nullptr;
-
-    // Find the respective SLAPrintObject.
-    if (m_print_object_idx < 0 || m_print_objects_count != int(print_objects.size())) {
-        m_print_objects_count = print_objects.size();
-        m_print_object_idx = -1;
-        for (const SLAPrintObject* po : print_objects) {
-            ++m_print_object_idx;
-            if (po->model_object()->id() == mo->id()) {
-                print_object = po;
-                break;
-            }
-        }
-    }
-
-    // If there is a valid SLAPrintObject, check state of Hollowing step.
-    if (print_object) {
-        if (print_object->is_step_done(slaposDrillHoles) && print_object->has_mesh(slaposDrillHoles)) {
-            size_t timestamp = print_object->step_state_with_timestamp(slaposDrillHoles).timestamp;
-            if (timestamp > m_old_hollowing_timestamp) {
-                const TriangleMesh& backend_mesh = print_object->get_mesh_to_slice();
-                if (! backend_mesh.empty()) {
-                    m_hollowed_mesh_transformed.reset(new TriangleMesh(backend_mesh));
-                    Transform3d trafo_inv = (canvas->sla_print()->sla_trafo(*mo) * print_object->model_object()->volumes.front()->get_transformation().get_matrix()).inverse();
-                    m_hollowed_mesh_transformed->transform(trafo_inv);
-                    m_drainholes = print_object->model_object()->sla_drain_holes;
-                    m_old_hollowing_timestamp = timestamp;
-
-                    indexed_triangle_set interior = print_object->hollowed_interior_mesh();
-                    its_flip_triangles(interior);
-                    m_hollowed_interior_transformed = std::make_unique<TriangleMesh>(std::move(interior));
-                    m_hollowed_interior_transformed->transform(trafo_inv);
-                }
-                else {
-                    m_hollowed_mesh_transformed.reset(nullptr);
-                }
-            }
-        }
-        else
-            m_hollowed_mesh_transformed.reset(nullptr);
-    }
-}
-
-
-void HollowedMesh::on_release()
-{
-    m_hollowed_mesh_transformed.reset();
-    m_old_hollowing_timestamp = 0;
-    m_print_object_idx = -1;
-}
-
-
-const TriangleMesh* HollowedMesh::get_hollowed_mesh() const
-{
-    return m_hollowed_mesh_transformed.get();
-}
-
-const TriangleMesh* HollowedMesh::get_hollowed_interior() const
-{
-    return m_hollowed_interior_transformed.get();
-}
-
-
-
-
 void Raycaster::on_update()
 {
     wxBusyCursor wait;
     const ModelObject* mo = get_pool()->selection_info()->model_object();
+    const ModelVolume* mv = get_pool()->selection_info()->model_volume();
 
-    if (! mo)
+    if (mo == nullptr && mv == nullptr)
         return;
 
+    std::vector<ModelVolume*> mvs;
+    if (mv != nullptr)
+        mvs.push_back(const_cast<ModelVolume*>(mv));
+    else
+        mvs = mo->volumes;
+
     std::vector<const TriangleMesh*> meshes;
-    const std::vector<ModelVolume*>& mvs = mo->volumes;
-    if (mvs.size() == 1) {
-        assert(mvs.front()->is_model_part());
-        const HollowedMesh* hollowed_mesh_tracker = get_pool()->hollowed_mesh();
-        if (hollowed_mesh_tracker && hollowed_mesh_tracker->get_hollowed_mesh())
-            meshes.push_back(hollowed_mesh_tracker->get_hollowed_mesh());
+    bool force_raycaster_regeneration = false;
+    if (wxGetApp().preset_bundle->printers.get_selected_preset().printer_technology() == ptSLA) {
+        // For sla printers we use the mesh generated by the backend
+        std::shared_ptr<const indexed_triangle_set> preview_mesh_ptr;
+        const SLAPrintObject* po = get_pool()->selection_info()->print_object();
+        if (po != nullptr)
+            preview_mesh_ptr = po->get_mesh_to_print();
+        else
+            preview_mesh_ptr.reset();
+
+        m_sla_mesh_cache = (preview_mesh_ptr != nullptr) ? TriangleMesh{ *preview_mesh_ptr } : TriangleMesh();
+
+        if (!m_sla_mesh_cache.empty()) {
+            m_sla_mesh_cache.transform(po->trafo().inverse());
+            meshes.emplace_back(&m_sla_mesh_cache);
+            force_raycaster_regeneration = true;
+        }
     }
+
     if (meshes.empty()) {
+        const std::vector<ModelVolume*>& mvs = mo->volumes;
         for (const ModelVolume* mv : mvs) {
             if (mv->is_model_part())
                 meshes.push_back(&mv->mesh());
         }
     }
 
-    if (meshes != m_old_meshes) {
+    if (force_raycaster_regeneration || meshes != m_old_meshes) {
         m_raycasters.clear();
         for (const TriangleMesh* mesh : meshes)
-            m_raycasters.emplace_back(new MeshRaycaster(*mesh));
+            m_raycasters.emplace_back(new MeshRaycaster(std::make_shared<const TriangleMesh>(*mesh)));
         m_old_meshes = meshes;
     }
 }
@@ -352,8 +309,9 @@ std::vector<const MeshRaycaster*> Raycaster::raycasters() const
     return mrcs;
 }
 
+} // namespace GUI
 
-
+namespace GUI {
 
 
 void ObjectClipper::on_update()
@@ -364,24 +322,42 @@ void ObjectClipper::on_update()
 
     // which mesh should be cut?
     std::vector<const TriangleMesh*> meshes;
-    bool has_hollowed = get_pool()->hollowed_mesh() && get_pool()->hollowed_mesh()->get_hollowed_mesh();
-    if (has_hollowed)
-        meshes.push_back(get_pool()->hollowed_mesh()->get_hollowed_mesh());
+    std::vector<Geometry::Transformation> trafos;
+    bool force_clipper_regeneration = false;
 
-    if (meshes.empty())
-        for (const ModelVolume* mv : mo->volumes)
-            meshes.push_back(&mv->mesh());
-
-    if (meshes != m_old_meshes) {
-        m_clippers.clear();
-        for (const TriangleMesh* mesh : meshes) {
-            m_clippers.emplace_back(new MeshClipper);
-            m_clippers.back()->set_mesh(*mesh);
+    std::unique_ptr<MeshClipper> mc;
+    Geometry::Transformation     mc_tr;
+    if (wxGetApp().preset_bundle->printers.get_selected_preset().printer_technology() == ptSLA) {
+        // For sla printers we use the mesh generated by the backend
+        const SLAPrintObject* po = get_pool()->selection_info()->print_object();
+        if (po) {
+            auto partstoslice = po->get_parts_to_slice();
+            if (! partstoslice.empty()) {
+                mc = std::make_unique<MeshClipper>();
+                mc->set_mesh(range(partstoslice));
+                mc_tr = Geometry::Transformation{po->trafo().inverse().cast<double>()};
+            }
         }
-        m_old_meshes = meshes;
+    }
 
-        if (has_hollowed)
-            m_clippers.front()->set_negative_mesh(*get_pool()->hollowed_mesh()->get_hollowed_interior());
+    if (!mc && meshes.empty()) {
+        for (const ModelVolume* mv : mo->volumes) {
+            meshes.emplace_back(&mv->mesh());
+            trafos.emplace_back(mv->get_transformation());
+        }
+    }
+
+    if (mc || force_clipper_regeneration || meshes != m_old_meshes) {
+        m_clippers.clear();
+        for (size_t i = 0; i < meshes.size(); ++i) {
+            m_clippers.emplace_back(new MeshClipper, trafos[i]);
+            m_clippers.back().first->set_mesh(meshes[i]->its);
+        }
+        m_old_meshes = std::move(meshes);
+
+        if (mc) {
+            m_clippers.emplace_back(std::move(mc), mc_tr);
+        }
 
         m_active_inst_bb_radius =
             mo->instance_bounding_box(get_pool()->selection_info()->get_active_instance()).radius();
@@ -398,35 +374,73 @@ void ObjectClipper::on_release()
 
 }
 
-void ObjectClipper::render_cut() const
+void ObjectClipper::render_cut(const std::vector<size_t>* ignore_idxs) const
 {
     if (m_clp_ratio == 0.)
         return;
     const SelectionInfo* sel_info = get_pool()->selection_info();
-    const ModelObject* mo = sel_info->model_object();
-    Geometry::Transformation inst_trafo = mo->instances[sel_info->get_active_instance()]->get_transformation();
+    const Geometry::Transformation inst_trafo = sel_info->model_object()->instances[sel_info->get_active_instance()]->get_transformation();
+    
+    std::vector<size_t> ignore_idxs_local = ignore_idxs ? *ignore_idxs : std::vector<size_t>();
 
-    size_t clipper_id = 0;
-    for (const ModelVolume* mv : mo->volumes) {
-        Geometry::Transformation vol_trafo  = mv->get_transformation();
-        Geometry::Transformation trafo = inst_trafo * vol_trafo;
+    for (auto& clipper : m_clippers) {
+        Geometry::Transformation trafo = inst_trafo * clipper.second;
         trafo.set_offset(trafo.get_offset() + Vec3d(0., 0., sel_info->get_sla_shift()));
-
-        auto& clipper = m_clippers[clipper_id];
-        clipper->set_plane(*m_clp);
-        clipper->set_transformation(trafo);
-        clipper->set_limiting_plane(ClippingPlane(Vec3d::UnitZ(), -SINKING_Z_THRESHOLD));
-        glsafe(::glPushMatrix());
-        glsafe(::glColor3f(1.0f, 0.37f, 0.0f));
-        clipper->render_cut();
-        glsafe(::glPopMatrix());
-
-        ++clipper_id;
+        clipper.first->set_plane(*m_clp);
+        clipper.first->set_transformation(trafo);
+        clipper.first->set_limiting_plane(ClippingPlane(Vec3d::UnitZ(), -SINKING_Z_THRESHOLD));      
+        clipper.first->render_cut({ 1.0f, 0.37f, 0.0f, 1.0f }, &ignore_idxs_local);
+        clipper.first->render_contour({ 1.f, 1.f, 1.f, 1.f },  &ignore_idxs_local);
+  
+        // Now update the ignore idxs. Find the first element belonging to the next clipper,
+        // and remove everything before it and decrement everything by current number of contours.
+        const int num_of_contours = clipper.first->get_number_of_contours();
+        ignore_idxs_local.erase(ignore_idxs_local.begin(), std::find_if(ignore_idxs_local.begin(), ignore_idxs_local.end(), [num_of_contours](size_t idx) { return idx >= size_t(num_of_contours); } ));
+        for (size_t& idx : ignore_idxs_local)
+            idx -= num_of_contours;
     }
 }
 
 
-void ObjectClipper::set_position(double pos, bool keep_normal)
+int ObjectClipper::get_number_of_contours() const
+{
+    int sum = 0;
+    for (const auto& [clipper, trafo] : m_clippers)
+        sum += clipper->get_number_of_contours();
+    return sum;
+}
+
+int ObjectClipper::is_projection_inside_cut(const Vec3d& point) const
+{
+    if (m_clp_ratio == 0.)
+        return -1;
+    int idx_offset = 0;
+    for (const auto& [clipper, trafo] : m_clippers) {
+        if (int idx = clipper->is_projection_inside_cut(point); idx != -1)
+            return idx_offset + idx;
+        idx_offset += clipper->get_number_of_contours();
+    }
+    return -1;
+}
+
+bool ObjectClipper::has_valid_contour() const
+{
+    return m_clp_ratio != 0. && std::any_of(m_clippers.begin(), m_clippers.end(), [](const auto& cl) { return cl.first->has_valid_contour(); });
+}
+
+std::vector<Vec3d> ObjectClipper::point_per_contour() const
+{
+    std::vector<Vec3d> pts;
+
+    for (const auto& clipper : m_clippers) {
+        const std::vector<Vec3d> pts_clipper = clipper.first->point_per_contour();
+        pts.insert(pts.end(), pts_clipper.begin(), pts_clipper.end());;
+    }
+    return pts;
+}
+
+
+void ObjectClipper::set_position_by_ratio(double pos, bool keep_normal)
 {
     const ModelObject* mo = get_pool()->selection_info()->model_object();
     int active_inst = get_pool()->selection_info()->get_active_instance();
@@ -444,6 +458,25 @@ void ObjectClipper::set_position(double pos, bool keep_normal)
     get_pool()->get_canvas()->set_as_dirty();
 }
 
+void ObjectClipper::set_range_and_pos(const Vec3d& cpl_normal, double cpl_offset, double pos)
+{
+    m_clp.reset(new ClippingPlane(cpl_normal, cpl_offset));
+    m_clp_ratio = pos;
+    get_pool()->get_canvas()->set_as_dirty();
+}
+
+const ClippingPlane* ObjectClipper::get_clipping_plane(bool ignore_hide_clipped) const
+{
+    static const ClippingPlane no_clip = ClippingPlane::ClipsNothing();
+    return (ignore_hide_clipped || m_hide_clipped) ? m_clp.get() : &no_clip;
+}
+
+void ObjectClipper::set_behavior(bool hide_clipped, bool fill_cut, double contour_width)
+{
+    m_hide_clipped = hide_clipped;
+    for (auto& clipper : m_clippers)
+        clipper.first->set_behaviour(fill_cut, contour_width);
+}
 
 
 void SupportsClipper::on_update()
@@ -453,86 +486,74 @@ void SupportsClipper::on_update()
     if (! mo || ! is_sla)
         return;
 
-    const GLCanvas3D* canvas = get_pool()->get_canvas();
-    const PrintObjects& print_objects = canvas->sla_print()->objects();
-    const SLAPrintObject* print_object = m_print_object_idx != -1
-            ? print_objects[m_print_object_idx]
-            : nullptr;
+    const SLAPrintObject* po = get_pool()->selection_info()->print_object();
+    if (po == nullptr)
+        return;
 
-    // Find the respective SLAPrintObject.
-    if (m_print_object_idx < 0 || m_print_objects_count != int(print_objects.size())) {
-        m_print_objects_count = print_objects.size();
-        m_print_object_idx = -1;
-        for (const SLAPrintObject* po : print_objects) {
-            ++m_print_object_idx;
-            if (po->model_object()->id() == mo->id()) {
-                print_object = po;
-                break;
-            }
-        }
+    if (po->get_mesh_to_print() == nullptr) {
+        // The object has been not sliced yet. We better dump the cached data.
+        m_supports_clipper.reset();
+        m_pad_clipper.reset();
+        return;
     }
 
-    if (print_object
-     && print_object->is_step_done(slaposSupportTree)
-     && ! print_object->support_mesh().empty())
-    {
-        // If the supports are already calculated, save the timestamp of the respective step
-        // so we can later tell they were recalculated.
-        size_t timestamp = print_object->step_state_with_timestamp(slaposSupportTree).timestamp;
-        if (! m_clipper || timestamp != m_old_timestamp) {
-            // The timestamp has changed.
-            m_clipper.reset(new MeshClipper);
-            // The mesh should already have the shared vertices calculated.
-            m_clipper->set_mesh(print_object->support_mesh());
-            m_old_timestamp = timestamp;
-        }
+    const TriangleMesh& support_mesh = po->support_mesh();
+    if (support_mesh.empty()) {
+        // The supports are not available yet. We better dump the cached data.
+        m_supports_clipper.reset();
+    }
+    else {
+        m_supports_clipper.reset(new MeshClipper);
+        m_supports_clipper->set_mesh(support_mesh.its);
+    }
+
+    const TriangleMesh& pad_mesh = po->pad_mesh();
+    if (pad_mesh.empty()) {
+        // The supports are not available yet. We better dump the cached data.
+        m_pad_clipper.reset();
+    }
+    else {
+        m_pad_clipper.reset(new MeshClipper);
+        m_pad_clipper->set_mesh(pad_mesh.its);
     }
-    else
-        // The supports are not valid. We better dump the cached data.
-        m_clipper.reset();
 }
 
 
 void SupportsClipper::on_release()
 {
-    m_clipper.reset();
-    m_old_timestamp = 0;
+    m_supports_clipper.reset();
+    m_pad_clipper.reset();
     m_print_object_idx = -1;
 }
 
 void SupportsClipper::render_cut() const
 {
     const CommonGizmosDataObjects::ObjectClipper* ocl = get_pool()->object_clipper();
-    if (ocl->get_position() == 0.
-     || ! get_pool()->instances_hider()->are_supports_shown()
-     || ! m_clipper)
+    if (ocl->get_position() == 0.)
         return;
 
+    const SLAPrintObject* po = get_pool()->selection_info()->print_object();
+    if (po == nullptr)
+        return;
+
+    Geometry::Transformation po_trafo(po->trafo());
+
     const SelectionInfo* sel_info = get_pool()->selection_info();
-    const ModelObject* mo = sel_info->model_object();
-    Geometry::Transformation inst_trafo = mo->instances[sel_info->get_active_instance()]->get_transformation();
-    //Geometry::Transformation vol_trafo  = mo->volumes.front()->get_transformation();
-    Geometry::Transformation trafo = inst_trafo;// * vol_trafo;
-    trafo.set_offset(trafo.get_offset() + Vec3d(0., 0., sel_info->get_sla_shift()));
+    Geometry::Transformation inst_trafo = sel_info->model_object()->instances[sel_info->get_active_instance()]->get_transformation();
+    inst_trafo = Geometry::Transformation(inst_trafo.get_matrix() * po_trafo.get_matrix().inverse());
+    inst_trafo.set_offset(inst_trafo.get_offset() + Vec3d(0.0, 0.0, sel_info->get_sla_shift()));
 
+    if (m_supports_clipper != nullptr) {
+        m_supports_clipper->set_plane(*ocl->get_clipping_plane());
+        m_supports_clipper->set_transformation(inst_trafo);
+        m_supports_clipper->render_cut({ 1.0f, 0.f, 0.37f, 1.0f });
+    }
 
-    // Get transformation of supports
-    Geometry::Transformation supports_trafo = trafo;
-    supports_trafo.set_scaling_factor(Vec3d::Ones());
-    supports_trafo.set_offset(Vec3d(trafo.get_offset()(0), trafo.get_offset()(1), sel_info->get_sla_shift()));
-    supports_trafo.set_rotation(Vec3d(0., 0., trafo.get_rotation()(2)));
-    // I don't know why, but following seems to be correct.
-    supports_trafo.set_mirror(Vec3d(trafo.get_mirror()(0) * trafo.get_mirror()(1) * trafo.get_mirror()(2),
-                                    1,
-                                    1.));
-
-    m_clipper->set_plane(*ocl->get_clipping_plane());
-    m_clipper->set_transformation(supports_trafo);
-
-    glsafe(::glPushMatrix());
-    glsafe(::glColor3f(1.0f, 0.f, 0.37f));
-    m_clipper->render_cut();
-    glsafe(::glPopMatrix());
+    if (m_pad_clipper != nullptr) {
+        m_pad_clipper->set_plane(*ocl->get_clipping_plane());
+        m_pad_clipper->set_transformation(inst_trafo);
+        m_pad_clipper->render_cut({ 0.6f, 0.f, 0.222f, 1.0f });
+    }
 }
 
 
