@@ -27,6 +27,8 @@
 #include "slic3r/GUI/MainFrame.hpp"
 #include "GUI_App.hpp"
 
+#include "Widgets/CheckBox.hpp"
+
 namespace Slic3r {
 namespace GUI {
 
@@ -83,6 +85,7 @@ void MsgDialog::SetButtonLabel(wxWindowID btn_id, const wxString& label, bool se
 wxButton* MsgDialog::add_button(wxWindowID btn_id, bool set_focus /*= false*/, const wxString& label/* = wxString()*/)
 {
     wxButton* btn = new wxButton(this, btn_id, label);
+    wxGetApp().SetWindowVariantForButton(btn);
     if (set_focus) {
         btn->SetFocus();
         // For non-MSW platforms SetFocus is not enought to use it as default, when the dialog is closed by ENTER
@@ -119,21 +122,20 @@ void MsgDialog::finalize()
     this->CenterOnParent();
 }
 
-
 // Text shown as HTML, so that mouse selection and Ctrl-V to copy will work.
-static void add_msg_content(wxWindow* parent, wxBoxSizer* content_sizer, wxString msg, bool monospaced_font = false, bool is_marked_msg = false)
+static void add_msg_content(MsgDialog* parent, wxBoxSizer* content_sizer, const HtmlContent& content)
 {
     wxHtmlWindow* html = new wxHtmlWindow(parent, wxID_ANY, wxDefaultPosition, wxDefaultSize, wxHW_SCROLLBAR_AUTO);
 
     // count lines in the message
     int msg_lines = 0;
-    if (!monospaced_font) {
+    if (!content.is_monospaced_font) {
         int line_len = 55;// count of symbols in one line
         int start_line = 0;
-        for (auto i = msg.begin(); i != msg.end(); ++i) {
+        for (auto i = content.msg.begin(); i != content.msg.end(); ++i) {
             if (*i == '\n') {
-                int cur_line_len = i - msg.begin() - start_line;
-                start_line = i - msg.begin();
+                int cur_line_len = i - content.msg.begin() - start_line;
+                start_line = i - content.msg.begin();
                 if (cur_line_len == 0 || line_len > cur_line_len)
                     msg_lines++;
                 else
@@ -172,11 +174,11 @@ static void add_msg_content(wxWindow* parent, wxBoxSizer* content_sizer, wxStrin
     }
 
     // if message containes the table
-    if (msg.Contains("<tr>")) {
-        int lines = msg.Freq('\n') + 1;
+    if (content.msg.Contains("<tr>")) {
+        int lines = content.msg.Freq('\n') + 1;
         int pos = 0;
-        while (pos < (int)msg.Len() && pos != wxNOT_FOUND) {
-            pos = msg.find("<tr>", pos + 1);
+        while (pos < (int)content.msg.Len() && pos != wxNOT_FOUND) {
+            pos = content.msg.find("<tr>", pos + 1);
             lines += 2;
         }
         int page_height = std::min(int(font.GetPixelSize().y+2) * lines, 68 * em);
@@ -184,16 +186,16 @@ static void add_msg_content(wxWindow* parent, wxBoxSizer* content_sizer, wxStrin
     }
     else {
         wxClientDC dc(parent);
-        wxSize msg_sz = dc.GetMultiLineTextExtent(msg);
+        wxSize msg_sz = dc.GetMultiLineTextExtent(content.msg);
         page_size = wxSize(std::min(msg_sz.GetX() + 2 * em, 68 * em),
                            std::min(msg_sz.GetY() + 2 * em, 68 * em));
     }
     html->SetMinSize(page_size);
 
-    std::string msg_escaped = xml_escape(into_u8(msg), is_marked_msg);
+    std::string msg_escaped = xml_escape(into_u8(content.msg), content.is_marked_msg || content.on_link_clicked);
     boost::replace_all(msg_escaped, "\r\n", "<br>");
     boost::replace_all(msg_escaped, "\n", "<br>");
-    if (monospaced_font)
+    if (content.is_monospaced_font)
         // Code formatting will be preserved. This is useful for reporting errors from the placeholder parser.
         msg_escaped = std::string("<pre><code>") + msg_escaped + "</code></pre>";
     html->SetPage(format_wxstr("<html>"
@@ -205,8 +207,13 @@ static void add_msg_content(wxWindow* parent, wxBoxSizer* content_sizer, wxStrin
                                "</html>", 
                     bgr_clr_str, text_clr_str, from_u8(msg_escaped)));
 
-    html->Bind(wxEVT_HTML_LINK_CLICKED, [parent](wxHtmlLinkEvent& event) {
-        wxGetApp().open_browser_with_warning_dialog(event.GetLinkInfo().GetHref(), parent, false);
+    html->Bind(wxEVT_HTML_LINK_CLICKED, [parent, &content](wxHtmlLinkEvent& event) {
+        if (content.on_link_clicked) {
+            parent->EndModal(wxID_CLOSE);
+            content.on_link_clicked(into_u8(event.GetLinkInfo().GetHref()));
+        }
+        else
+            wxGetApp().open_browser_with_warning_dialog(event.GetLinkInfo().GetHref(), parent, false);
         event.Skip(false);
     });
 
@@ -216,21 +223,44 @@ static void add_msg_content(wxWindow* parent, wxBoxSizer* content_sizer, wxStrin
 
 // ErrorDialog
 
-ErrorDialog::ErrorDialog(wxWindow *parent, const wxString &msg, bool monospaced_font)
-    : MsgDialog(parent, wxString::Format(_(L("%s error")), SLIC3R_APP_NAME), 
-                        wxString::Format(_(L("%s has encountered an error")), SLIC3R_APP_NAME), wxOK)
-	, msg(msg)
+void ErrorDialog::create(const HtmlContent& content, int icon_width)
 {
-    add_msg_content(this, content_sizer, msg, monospaced_font);
+    add_msg_content(this, content_sizer, content);
 
-	// Use a small bitmap with monospaced font, as the error text will not be wrapped.
-	logo->SetBitmap(*get_bmp_bundle("PrusaSlicer_192px_grayscale.png", monospaced_font ? 48 : /*1*/84));
+    // Use a small bitmap with monospaced font, as the error text will not be wrapped.
+    logo->SetBitmap(*get_bmp_bundle("PrusaSlicer_192px_grayscale.png", icon_width));
 
     SetMaxSize(wxSize(-1, CONTENT_MAX_HEIGHT*wxGetApp().em_unit()));
 
     finalize();
 }
 
+ErrorDialog::ErrorDialog(wxWindow *parent, const wxString &msg, bool monospaced_font)
+    : MsgDialog(parent, wxString::Format(_L("%s error"), SLIC3R_APP_NAME), 
+                        wxString::Format(_L("%s has encountered an error"), SLIC3R_APP_NAME), wxOK)
+    , m_content(HtmlContent{ msg, monospaced_font, true })
+{
+    create(m_content, monospaced_font ? 48 : 84);
+}
+
+ErrorDialog::ErrorDialog(wxWindow *parent, const wxString &msg, const std::function<void(const std::string&)> &on_link_clicked)
+    : MsgDialog(parent, wxString::Format(_L("%s error"), SLIC3R_APP_NAME), 
+                        wxString::Format(_L("%s has encountered an error"), SLIC3R_APP_NAME), wxOK)
+    , m_content(HtmlContent{ msg, false, true, on_link_clicked })
+{
+    create(m_content, 84);
+}
+
+
+HtmlCapableRichMessageDialog::HtmlCapableRichMessageDialog(wxWindow                                       *parent,
+                                                           const wxString                                 &msg,
+                                                           const wxString                                 &caption,
+                                                           long                                           style,
+                                                           const std::function<void(const std::string &)> &on_link_clicked)
+    : RichMessageDialogBase(parent, HtmlContent{msg, false, true, on_link_clicked}, caption, style)
+{}
+
+
 // WarningDialog
 
 WarningDialog::WarningDialog(wxWindow *parent,
@@ -240,7 +270,7 @@ WarningDialog::WarningDialog(wxWindow *parent,
     : MsgDialog(parent, caption.IsEmpty() ? wxString::Format(_L("%s warning"), SLIC3R_APP_NAME) : caption, 
                         wxString::Format(_L("%s has a warning")+":", SLIC3R_APP_NAME), style)
 {
-    add_msg_content(this, content_sizer, message);
+    add_msg_content(this, content_sizer, HtmlContent{ message });
     finalize();
 }
 
@@ -253,41 +283,55 @@ MessageDialog::MessageDialog(wxWindow* parent,
     long style/* = wxOK*/)
     : MsgDialog(parent, caption.IsEmpty() ? wxString::Format(_L("%s info"), SLIC3R_APP_NAME) : caption, wxEmptyString, style)
 {
-    add_msg_content(this, content_sizer, get_wraped_wxString(message));
+    add_msg_content(this, content_sizer, HtmlContent{ get_wraped_wxString(message) });
     finalize();
 }
+#endif
 
 
-// RichMessageDialog
+// RichMessageDialogBase
 
-RichMessageDialog::RichMessageDialog(wxWindow* parent,
+RichMessageDialogBase::RichMessageDialogBase(wxWindow* parent,
     const wxString& message,
     const wxString& caption/* = wxEmptyString*/,
     long style/* = wxOK*/)
+    : RichMessageDialogBase(parent, HtmlContent{get_wraped_wxString(message)}, caption, style)
+{}
+
+RichMessageDialogBase::RichMessageDialogBase(wxWindow* parent, const HtmlContent& content, const wxString& caption, long style)
     : MsgDialog(parent, caption.IsEmpty() ? wxString::Format(_L("%s info"), SLIC3R_APP_NAME) : caption, wxEmptyString, style)
 {
-    add_msg_content(this, content_sizer, get_wraped_wxString(message));
+    m_content = content; // We need a copy for the on_link_clicked lambda.
+    add_msg_content(this, content_sizer, m_content);
 
+#ifdef _WIN32 // See comment in the header where m_checkBox is defined.
+    m_checkBox = new ::CheckBox(this, m_checkBoxText);
+#else
     m_checkBox = new wxCheckBox(this, wxID_ANY, m_checkBoxText);
+#endif
+
     wxGetApp().UpdateDarkUI(m_checkBox);
     m_checkBox->Bind(wxEVT_CHECKBOX, [this](wxCommandEvent&) { m_checkBoxValue = m_checkBox->GetValue(); });
 
     btn_sizer->Insert(0, m_checkBox, wxALIGN_CENTER_VERTICAL);
 
-    finalize();
+    finalize();    
 }
 
-int RichMessageDialog::ShowModal()
+
+int RichMessageDialogBase::ShowModal()
 {
     if (m_checkBoxText.IsEmpty())
         m_checkBox->Hide();
-    else
+    else {
         m_checkBox->SetLabelText(m_checkBoxText);
+        m_checkBox->Update();
+    }
     Layout();
 
     return wxDialog::ShowModal();
 }
-#endif
+
 
 // InfoDialog
 
@@ -295,7 +339,7 @@ InfoDialog::InfoDialog(wxWindow* parent, const wxString &title, const wxString&
 	: MsgDialog(parent, wxString::Format(_L("%s information"), SLIC3R_APP_NAME), title, style)
 	, msg(msg)
 {
-    add_msg_content(this, content_sizer, msg, false, is_marked_msg);
+    add_msg_content(this, content_sizer, HtmlContent{ msg, false, is_marked_msg });
     finalize();
 }
 
