@@ -184,7 +184,7 @@ PhysicalPrinterDialog::PhysicalPrinterDialog(wxWindow* parent, wxString printer_
     m_add_preset_btn->SetToolTip(_L("Add preset for this printer device")); 
     m_add_preset_btn->Bind(wxEVT_BUTTON, &PhysicalPrinterDialog::AddPreset, this);
 
-    m_printer_name    = new wxTextCtrl(this, wxID_ANY, printer_name, wxDefaultPosition, wxDefaultSize);
+    m_printer_name    = new ::TextInput(this,printer_name);
     wxGetApp().UpdateDarkUI(m_printer_name);
     m_printer_name->Bind(wxEVT_TEXT, [this](wxEvent&) { this->update_full_printer_names(); });
 
@@ -242,7 +242,7 @@ PhysicalPrinterDialog::PhysicalPrinterDialog(wxWindow* parent, wxString printer_
 
     if (new_printer) {
         m_printer_name->SetFocus();
-        m_printer_name->SelectAll();
+        m_printer_name->GetTextCtrl()->SelectAll();
     }
 
     this->Fit();
@@ -436,11 +436,15 @@ void PhysicalPrinterDialog::build_printhost_settings(ConfigOptionsGroup* m_optgr
 
     m_optgroup->activate();
 
+    const auto opt = m_config->option<ConfigOptionEnum<PrintHostType>>("host_type");
+    m_last_host_type = opt->value;
+    m_opened_as_connect = (m_last_host_type == htPrusaConnect);
+
     Field* printhost_field = m_optgroup->get_field("print_host");
     if (printhost_field)
     {
-        wxTextCtrl* temp = dynamic_cast<wxTextCtrl*>(printhost_field->getWindow());
-        if (temp)
+        text_ctrl* temp = dynamic_cast<text_ctrl*>(printhost_field->getWindow());
+        if (temp) {
             temp->Bind(wxEVT_TEXT, ([printhost_field, temp](wxEvent& e)
             {
 #ifndef __WXGTK__
@@ -457,6 +461,7 @@ void PhysicalPrinterDialog::build_printhost_settings(ConfigOptionsGroup* m_optgr
                 if (field)
                     field->propagate_value();
             }), temp->GetId());
+        }
     }
 
     // Always fill in the "printhost_port" combo box from the config and select it.
@@ -488,12 +493,6 @@ void PhysicalPrinterDialog::update(bool printer_change)
         const auto opt = m_config->option<ConfigOptionEnum<PrintHostType>>("host_type");
         m_optgroup->show_field("host_type");
 
-        // hide PrusaConnect address
-        if (Field* printhost_field = m_optgroup->get_field("print_host"); printhost_field) {
-            if (wxTextCtrl* temp = dynamic_cast<wxTextCtrl*>(printhost_field->getWindow()); temp && temp->GetValue() == L"https://connect.prusa3d.com") {
-                temp->SetValue(wxString());
-            }
-        }
         if (opt->value == htPrusaLink) { // PrusaConnect does NOT allow http digest
             m_optgroup->show_field("printhost_authorization_type");
             AuthorizationType auth_type = m_config->option<ConfigOptionEnum<AuthorizationType>>("printhost_authorization_type")->value;
@@ -506,15 +505,30 @@ void PhysicalPrinterDialog::update(bool printer_change)
             for (const std::string& opt_key : std::vector<std::string>{ "printhost_user", "printhost_password" })
                 m_optgroup->hide_field(opt_key);
             supports_multiple_printers = opt && opt->value == htRepetier;
-            if (opt->value == htPrusaConnect) { // automatically show default prusaconnect address
-                if (Field* printhost_field = m_optgroup->get_field("print_host"); printhost_field) {
-                    if (wxTextCtrl* temp = dynamic_cast<wxTextCtrl*>(printhost_field->getWindow()); temp && temp->GetValue().IsEmpty()) {
-                        temp->SetValue(L"https://connect.prusa3d.com");
-                    }
-                }
+        }
+        // Hide Browse and Test buttons for Connect
+        if (opt->value == htPrusaConnect) {
+            m_printhost_browse_btn->Hide();
+            // hide show hostname and PrusaConnect address
+            Field* printhost_field = m_optgroup->get_field("print_host");
+            text_ctrl* printhost_win = printhost_field ? dynamic_cast<text_ctrl*>(printhost_field->getWindow()) : nullptr;
+            if (!m_opened_as_connect && printhost_win && m_last_host_type != htPrusaConnect){
+                m_stored_host = printhost_win->GetValue();
+                printhost_win->SetValue(L"https://connect.prusa3d.com");
+            }
+        } else {
+            m_printhost_browse_btn->Show();
+            // hide PrusaConnect address and show hostname
+            Field* printhost_field = m_optgroup->get_field("print_host");
+            text_ctrl* printhost_win = printhost_field ? dynamic_cast<text_ctrl*>(printhost_field->getWindow()) : nullptr;
+            if (!m_opened_as_connect && printhost_win && m_last_host_type == htPrusaConnect) {
+                wxString temp_host = printhost_win->GetValue();
+                printhost_win->SetValue(m_stored_host);
+                m_stored_host = temp_host;
             }
         }
         
+        m_last_host_type = opt->value;
     }
     else {
         m_optgroup->set_value("host_type", int(PrintHostType::htOctoPrint), false);
@@ -558,10 +572,13 @@ void PhysicalPrinterDialog::update_host_type(bool printer_change)
                 || boost::starts_with(model, "XL")
                 );
     };
-    // allowed models are: all MK3/S and MK2.5/S
+    // allowed models are: all MK3/S and MK2.5/S. 
+    // Since 2.6.2 also MINI, which makes list of supported printers same for both services.
+    // Lets keep these 2 functions separated for now.
     auto model_supports_prusaconnect = [](const std::string& model) {
         return model.size() >= 2 &&
                 ((boost::starts_with(model, "MK") && model[2] > '2' && model[2] <= '9')
+                || boost::starts_with(model, "MINI")
                 || boost::starts_with(model, "MK2.5")
                 || boost::starts_with(model, "XL")
                 );
@@ -671,7 +688,7 @@ void PhysicalPrinterDialog::update_full_printer_names()
             InfoDialog(this, format_wxstr("%1%: \"%2%\" ", _L("Unexpected character"),  str), 
                        _L("The following characters are not allowed in the name") + ": " + unusable_symbols).ShowModal();
             m_printer_name->SetValue(printer_name);
-            m_printer_name->SetInsertionPointEnd();
+            m_printer_name->GetTextCtrl()->SetInsertionPointEnd();
             return;
         }
     }
@@ -728,6 +745,20 @@ void PhysicalPrinterDialog::OnOK(wxEvent& event)
         return;
     }
 
+    Field* printhost_field = m_optgroup->get_field("print_host");
+    text_ctrl* printhost_win = printhost_field ? dynamic_cast<text_ctrl*>(printhost_field->getWindow()) : nullptr;
+    const auto opt = m_config->option<ConfigOptionEnum<PrintHostType>>("host_type");
+    if (opt->value == htPrusaConnect) {
+        if (printhost_win && printhost_win->GetValue() != L"https://connect.prusa3d.com"){
+            InfoDialog msg(this, _L("Warning"), _L("URL of PrusaConnect is different from https://connect.prusa3d.com. Do you want to continue?"), true, wxYES_NO);
+            if(msg.ShowModal() != wxID_YES){
+                printhost_win->SetValue(L"https://connect.prusa3d.com");
+                return;
+            }
+        }
+    }
+    
+
     PhysicalPrinterCollection& printers = wxGetApp().preset_bundle->physical_printers;
     const PhysicalPrinter* existing = printers.find_printer(into_u8(printer_name), false);
     if (existing && into_u8(printer_name) != printers.get_selected_printer_name())
@@ -835,5 +866,4 @@ void PhysicalPrinterDialog::DeletePreset(PresetForPrinter* preset_for_printer)
 
     update_host_type(true);
 }
-
 }}    // namespace Slic3r::GUI
