@@ -1,9 +1,14 @@
+///|/ Copyright (c) Prusa Research 2021 - 2023 Oleksandra Iushchenko @YuSanka, Lukáš Matěna @lukasmatena, Enrico Turri @enricoturri1966, Vojtěch Bubník @bubnikv
+///|/
+///|/ PrusaSlicer is released under the terms of the AGPLv3 or higher
+///|/
 #include "SendSystemInfoDialog.hpp"
 
 #include "libslic3r/AppConfig.hpp"
 #include "libslic3r/BlacklistedLibraryCheck.hpp"
 #include "libslic3r/Platform.hpp"
 #include "libslic3r/Utils.hpp"
+#include "libslic3r/Color.hpp"
 
 #include "slic3r/GUI/format.hpp"
 #include "slic3r/Utils/Http.hpp"
@@ -54,8 +59,10 @@ static const std::string SEND_SYSTEM_INFO_DOMAIN = "prusa3d.com";
 static const std::string SEND_SYSTEM_INFO_URL = "https://files." + SEND_SYSTEM_INFO_DOMAIN + "/wp-json/v1/ps";
 
 
+#if !ENABLE_GL_CORE_PROFILE
 // Declaration of a free function defined in OpenGLManager.cpp:
 std::string gl_get_string_safe(GLenum param, const std::string& default_value);
+#endif // !ENABLE_GL_CORE_PROFILE
 
 
 // A dialog with the information text and buttons send/dont send/ask later.
@@ -140,7 +147,7 @@ static bool check_internet_connection_win()
 {
     bool internet = true; // return true if COM object creation fails.
 
-    if (CoInitializeEx(NULL, COINIT_APARTMENTTHREADED) == S_OK) {
+    if (SUCCEEDED(CoInitializeEx(NULL, COINIT_APARTMENTTHREADED))) {
         {
             CComPtr<INetworkListManager> pNLM;
             if (pNLM.CoCreateInstance(CLSID_NetworkListManager) == S_OK) {
@@ -431,7 +438,7 @@ static std::string generate_system_info_json()
 
     pt::ptree hw_node;
     {
-        hw_node.put("ArchName", wxPlatformInfo::Get().GetArchName());
+        hw_node.put("ArchName", wxPlatformInfo::Get().GetBitnessName());
         size_t num = std::round(Slic3r::total_physical_memory()/107374100.);
         hw_node.put("RAM_GiB", std::to_string(num / 10) + "." + std::to_string(num % 10));
     }
@@ -501,14 +508,18 @@ static std::string generate_system_info_json()
 #endif // _WIN32
 
     pt::ptree opengl_node;
-    opengl_node.put("Version", OpenGLManager::get_gl_info().get_version());
-    opengl_node.put("GLSLVersion", OpenGLManager::get_gl_info().get_glsl_version());
+    opengl_node.put("Version", OpenGLManager::get_gl_info().get_version_string());
+    opengl_node.put("GLSLVersion", OpenGLManager::get_gl_info().get_glsl_version_string());
     opengl_node.put("Vendor", OpenGLManager::get_gl_info().get_vendor());
     opengl_node.put("Renderer", OpenGLManager::get_gl_info().get_renderer());
     // Generate list of OpenGL extensions:
+#if ENABLE_GL_CORE_PROFILE
+    std::vector<std::string> extensions_list = OpenGLManager::get_gl_info().get_extensions_list();
+#else
     std::string extensions_str = gl_get_string_safe(GL_EXTENSIONS, "");
     std::vector<std::string> extensions_list;
     boost::split(extensions_list, extensions_str, boost::is_any_of(" "), boost::token_compress_off);
+#endif // ENABLE_GL_CORE_PROFILE
     std::sort(extensions_list.begin(), extensions_list.end());
     pt::ptree extensions_node;
     for (const std::string& s : extensions_list) {
@@ -571,9 +582,8 @@ SendSystemInfoDialog::SendSystemInfoDialog(wxWindow* parent)
     wxColour bgr_clr = wxGetApp().get_window_default_clr();
     SetBackgroundColour(bgr_clr);
     const auto text_clr = wxGetApp().get_label_clr_default();
-    auto text_clr_str = wxString::Format(wxT("#%02X%02X%02X"), text_clr.Red(), text_clr.Green(), text_clr.Blue());
-    auto bgr_clr_str = wxString::Format(wxT("#%02X%02X%02X"), bgr_clr.Red(), bgr_clr.Green(), bgr_clr.Blue());
-
+    auto text_clr_str = encode_color(ColorRGB(text_clr.Red(), text_clr.Green(), text_clr.Blue()));
+    auto bgr_clr_str = encode_color(ColorRGB(bgr_clr.Red(), bgr_clr.Green(), bgr_clr.Blue()));
 
     auto *topSizer = new wxBoxSizer(wxVERTICAL);
     auto *vsizer = new wxBoxSizer(wxVERTICAL);
@@ -652,7 +662,7 @@ SendSystemInfoDialog::SendSystemInfoDialog(wxWindow* parent)
                                         wxString message;
                                         bool success = send_info(message);
                                         if (! message.IsEmpty())
-                                            InfoDialog(nullptr, wxEmptyString, message).ShowModal();
+                                            InfoDialog(this, wxEmptyString, message).ShowModal();
                                         if (success) {
                                             save_version();
                                             EndModal(0);
