@@ -30,7 +30,7 @@ public:
 	~PresetUpdater();
 
 	// If either version check or config updating is enabled, get the appropriate data in the background and cache it.
-	void sync(const PresetBundle *preset_bundle);
+	void sync(const PresetBundle *preset_bundle, wxEvtHandler* evt_handler);
 	void cancel_sync();
 
 	// If version check is enabled, check if chaced online slic3r version is newer, notify if so.
@@ -74,5 +74,6 @@ private:
 
 //wxDECLARE_EVENT(EVT_SLIC3R_VERSION_ONLINE, wxCommandEvent);
 //wxDECLARE_EVENT(EVT_SLIC3R_EXPERIMENTAL_VERSION_ONLINE, wxCommandEvent);
+wxDECLARE_EVENT(EVT_CONFIG_UPDATER_SYNC_DONE, wxCommandEvent);
 }
 #endif
