@@ -1,3 +1,7 @@
+///|/ Copyright (c) Prusa Research 2019 - 2022 Enrico Turri @enricoturri1966, Vojtěch Bubník @bubnikv, Lukáš Matěna @lukasmatena, Oleksandra Iushchenko @YuSanka
+///|/
+///|/ PrusaSlicer is released under the terms of the AGPLv3 or higher
+///|/
 #include "UndoRedo.hpp"
 
 #include <algorithm>
@@ -21,6 +25,8 @@
 #include <libslic3r/ObjectID.hpp>
 #include <libslic3r/Utils.hpp>
 
+#include "slic3r/GUI/3DScene.hpp"
+
 #include <boost/foreach.hpp>
 
 #ifndef NDEBUG
